package bench.gridm

import joins._
import events._
import scala.concurrent.ops._
import scala.collection.mutable.ListBuffer
import scala.compat.Platform

class Gridm(val count:Long) extends Joins{
  val className={
    val cn=this.getClass().getName()
    val from=cn.lastIndexOf(".")
    if (from>0)
      cn.substring(from+1)
    else
      cn
  }
  var cnt:Long=0

  def avg(arg1:Long,arg2:Long,arg3:Long)=(arg1+arg2+arg3)/3

  def consider(arg:(Long,Long,Long),fun1:(Long)=>Unit,fun2:()=>Unit):Unit={
    val m=avg(arg._1,arg._2,arg._3)
    cnt+=1
    if (cnt<count) {
      if (m>0) fun1(m-1)
    }
    else
      fun2()
  }

  object cbA extends AsyncEvent[Long]
  object cbB extends AsyncEvent[Long]
  object cbC extends AsyncEvent[Long]
  object cbD extends AsyncEvent[Long]
  object blockBegin extends SyncEvent[Unit,Long]
  object ready extends      NullaryAsyncEvent

  def block={
    blockBegin(Platform.currentTime)
  }

  protected def finished(a:Long):Unit={
    val ended=Platform.currentTime
    val begin=a
    val timed=ended-begin
    println(className+" "+count+" in "+timed+"ms  normalized ms/1K %3.3f".format(1000.0*timed/count)+"  throughput(/s): %9.2f".format(1000.0*count/timed))
    done()
  }





  join {
    case cbA(l1) and1 cbB(l2) and2 cbC(l3)  => {
      consider((l1,l2,l3),((a:Long)=>cbD(a)),(()=>ready()))     
    }
    case cbD(l1) and3 cbA(l2) and4 cbB(l3)  => {
      consider((l1,l2,l3),((a:Long)=>cbC(a)),(()=>ready()))
    }
    case cbC(l1) and5 cbD(l2) and6 cbA(l3)  => {
      consider((l1,l2,l3),((a:Long)=>cbB(a)),(()=>ready()))
    }
    case cbB(l1) and7 cbC(l2) and8 cbD(l3)  => {
      consider((l1,l2,l3),((a:Long)=>cbA(a)),(()=>ready()))
    }
    case blockBegin(b) and9 ready() => {
      finished(b)
      blockBegin.reply()
    }
  }


  var doneHandlers=new ListBuffer[()=>Unit]
  def addDone(hndl:()=>Unit)={
    doneHandlers+=hndl
  }
  def done()=doneHandlers.foreach(x=>x())
}

class Emitter(val g:Gridm,val fun:(Long)=>Unit,name:String) extends Thread(name) {
  var halted:Boolean=false
  def halt():Unit={halted=true  }
  override def run()={
    var cycle=0
    while (!halted) {
      fun(35)
      cycle+=1
      if (cycle%20==0)
	Thread.sleep(1)
    }
  }
  g.addDone(halt _)
}

class Cycle(val count:Long) {
  val g=new Gridm(count)
  var begin:Long=0
  var ended:Long=0
  val l=List(
    new Emitter(g,((arg:Long)=>g.cbA(arg)),"cbA"),
    new Emitter(g,((arg:Long)=>g.cbB(arg)),"cbB"),
    new Emitter(g,((arg:Long)=>g.cbC(arg)),"cbC"),
    new Emitter(g,((arg:Long)=>g.cbD(arg)),"cbD")
  )

  def start={
    System.gc
    l.foreach(e => e.start())
    g.block
    l.foreach(e => e.halt())
  }
}

object R_EventGrid {
  def main(arg:Array[String])={
    var iters:Long=30000
    if (arg.length>0)
      iters=arg(0).toLong
    else
      println("specify number of iterations as argument, using now "+iters)
	     (1 to 8).foreach (i=>{
	       Thread.sleep(1000)
	       val c=new Cycle(iters)
	       c.start
	     })
  }
}
