package bench

import scala.compat.Platform

trait FSM_abs {
  val ev_count:Long

  val className={
    val cn=this.getClass().getName()
    val from=cn.lastIndexOf(".")
    if (from>0)
      cn.substring(from+1)
    else
      cn
  }

  def block_impl():(Long)=>Long
  def up_impl():(Long)=>Unit
  def down_impl():(Long)=>Unit
  
  def waitUntilReady(tellDone:List[()=>Unit]) {
    val blck=block_impl()
    val timed=blck(Platform.currentTime)
    println(className+" "+ev_count+" in "+timed+"ms  normalized ms/1K "+(1000.0*timed/ev_count)+"  throughput(/s): "+(1000.0*ev_count/timed))
    tellDone.foreach(x => x())
  }
}

class FsmThread(val iterations:Long,val task:(Long)=>Unit) extends Thread {
  private var halted=false
  def halt()={halted=true}
  override def run()={
    var cycle:Long=0
    try {
    while (!halted && cycle<iterations) {
      task(cycle)
     if (cycle % 100==0)
	Thread.sleep(1)
      cycle+=1
    }
    } catch {
      case e:Throwable => println("halted "+cycle+" ")
    }
  }
}


class Benchmarker {
  val iterations=8
  val count=14000
  def cycle(fsm:FSM_abs)={
    val count=fsm.ev_count
    val up=fsm.up_impl()
    val down=fsm.down_impl()
    val threads=List(
      new FsmThread(count+1,((arg:Long)=>(up(arg)))),
      new FsmThread(count+1,((arg:Long)=>(down(arg)))))
    val tellDone=threads.map(x => (()=>x.halt()))
    System.gc()
    Thread.sleep(1000)
    threads.foreach(x => x.start())
    fsm.waitUntilReady(tellDone)
  }
}

