package bench_rsp.rsp.arena

import scala.events.{ImperativeAsyncEvent,ImperativeEvent}
import scala.events.sugarjoin.{mkDisjunction,mkChord}
import joins._
import bench_rsp.rsp.Constants
import bench_rsp.rsp.players.Player



object Arena {
  def create(code:Int,a:Player,b:Player,runs:Int)=code match {
    case  0=>new Arena_SJ_opt(a,b,runs)
    case  1=>new Arena_SJ_Unit(a,b,runs)
    case  2=>new Arena_SJ_Int(a,b,runs)
    case  3=>new Arena_JE_Int(a,b,runs,0)
    case  4=>new Arena_JE_Int(a,b,runs,1)

    case  5=>new Arena_SJ_S_opt(a,b,runs)
    case  6=>new Arena_SJ_S_Unit(a,b,runs)
    case  7=>new Arena_JE_S_Unit(a,b,runs)
    case  8=>new Arena_SJ_S_Int(a,b,runs)
    case  9=>new Arena_JE_S_Int(a,b,runs)

    case 10=>new Arena_JE_Unit(a,b,runs,false,true ,0)
    case 11=>new Arena_JE_Unit(a,b,runs,true,false ,0)
    case 12=>new Arena_JE_Unit(a,b,runs,true,true  ,0)
    case 13=>new Arena_JE_Unit(a,b,runs,false,false,0)

    case 14=>new Arena_JE_Unit(a,b,runs,false,true ,1)
    case 15=>new Arena_JE_Unit(a,b,runs,true,false ,1)
    case 16=>new Arena_JE_Unit(a,b,runs,true,true  ,1)
    case 17=>new Arena_JE_Unit(a,b,runs,false,false,1)
      
    case _=>throw new IllegalArgumentException("wrong code "+code)
  }

}

abstract class Arena(val a:Player,val b:Player,val runs:Integer,val asyncProviderType:Int) {
  val name=this.getClass
  var myRuns=0
  var endTime:Long=0
  var finished:Boolean=false
  scala.events.AsyncProvider.setProviderType(asyncProviderType)
  def moveHandler():Unit=synchronized {
    myRuns+=1
    if (!finished && myRuns>=runs) {
      endTime=scala.compat.Platform.currentTime
      finished=true
      println("Done"+Thread.currentThread)
      disable()
    }
  }

  a.setArena(this)
  b.setArena(this)
  if (!a.playerA) throw new IllegalArgumentException("player A lacks the playerA flag")
  if (b.playerA) throw new IllegalArgumentException("player B has the playerA flag")

  def arock(v:Int):Unit
  def ascissors(v:Int):Unit
  def apaper(v:Int):Unit
  def brock(v:Int):Unit
  def bscissors(v:Int):Unit
  def bpaper(v:Int):Unit
  def disable():Unit={}
  def asString=arenaName+" Arena "+runs
  def arenaName:String
  def asyncProvider=List("[FullThread]","[ThreadPool]")(asyncProviderType)
  println(asString)

}

class Arena_JE_Int(a:Player,b:Player,runs:Int,asyncProviderType:Int) extends Arena(a,b,runs,asyncProviderType) {
  val ev_arock=new ImperativeAsyncEvent[Int]
  val ev_ascissors=new ImperativeAsyncEvent[Int]
  val ev_apaper=new ImperativeAsyncEvent[Int]
  val ev_brock=new ImperativeAsyncEvent[Int]
  val ev_bscissors=new ImperativeAsyncEvent[Int]
  val ev_bpaper=new ImperativeAsyncEvent[Int]

  def arock(arg:Int)=ev_arock(arg)
  def ascissors(arg:Int)=ev_ascissors(arg)
  def apaper(arg:Int)=ev_apaper(arg)

  def brock(arg:Int)=ev_brock(arg)
  def bscissors(arg:Int)=ev_bscissors(arg)
  def bpaper(arg:Int)=ev_bpaper(arg)


  val (d1,a1,b1,b2,d2,a2,a3,b3,d3)=mkDisjunction(
    mkChord(ev_arock,   ev_brock    )  ,
    mkChord(ev_arock,   ev_bscissors )  ,
    mkChord(ev_arock,   ev_bpaper   )  ,
    mkChord(ev_ascissors,ev_brock    )  ,
    mkChord(ev_ascissors,ev_bscissors )  ,
    mkChord(ev_ascissors,ev_bpaper   )  ,
    mkChord(ev_apaper,  ev_brock    )  ,
    mkChord(ev_apaper,  ev_bscissors )  ,
    mkChord(ev_apaper,  ev_bpaper   )  )
  val drw=(d1 || d2 || d3).dropParam
  val wina=(a1 || a2 || a3).dropParam
  val winb=(b1 || b2 || b3).dropParam
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)
  override def disable():Unit=d1.disable()

  def arenaName="JES_A.Int."+asyncProvider
}

class Arena_JE_S_Int(a:Player,b:Player,runs:Int) extends Arena(a,b,runs,0) {
  val ev_arock=new ImperativeEvent[Int]
  val ev_ascissors=new ImperativeEvent[Int]
  val ev_apaper=new ImperativeEvent[Int]
  val ev_brock=new ImperativeEvent[Int]
  val ev_bscissors=new ImperativeEvent[Int]
  val ev_bpaper=new ImperativeEvent[Int]

  def arock(arg:Int)=ev_arock(arg)
  def ascissors(arg:Int)=ev_ascissors(arg)
  def apaper(arg:Int)=ev_apaper(arg)

  def brock(arg:Int)=ev_brock(arg)
  def bscissors(arg:Int)=ev_bscissors(arg)
  def bpaper(arg:Int)=ev_bpaper(arg)


  val (d1,a1,b1,b2,d2,a2,a3,b3,d3)=mkDisjunction(
    mkChord(ev_arock,   ev_brock    )  ,
    mkChord(ev_arock,   ev_bscissors )  ,
    mkChord(ev_arock,   ev_bpaper   )  ,

    mkChord(ev_ascissors,ev_brock    )  ,
    mkChord(ev_ascissors,ev_bscissors )  ,
    mkChord(ev_ascissors,ev_bpaper   )  ,

    mkChord(ev_apaper,  ev_brock    )  ,
    mkChord(ev_apaper,  ev_bscissors )  ,
    mkChord(ev_apaper,  ev_bpaper   )  )
  val drw=(d1 || d2 || d3).dropParam
  val wina=(a1 || a2 || a3).dropParam
  val winb=(b1 || b2 || b3).dropParam
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)
  override def disable():Unit=d1.disable()
  def arenaName="JES_S.Int"
}


class Arena_JE_Unit(a:Player,b:Player,runs:Int,optimize_nullary:Boolean,optimize_async:Boolean,asyncProviderType:Int) extends Arena(a,b,runs,asyncProviderType) {
  val ev_arock=new ImperativeAsyncEvent[Unit]
  val ev_ascissors=new ImperativeAsyncEvent[Unit]
  val ev_apaper=new ImperativeAsyncEvent[Unit]
  val ev_brock=new ImperativeAsyncEvent[Unit]
  val ev_bscissors=new ImperativeAsyncEvent[Unit]
  val ev_bpaper=new ImperativeAsyncEvent[Unit]

  def arock(arg:Int)=ev_arock()
  def ascissors(arg:Int)=ev_ascissors()
  def apaper(arg:Int)=ev_apaper()

  def brock(arg:Int)=ev_brock()
  def bscissors(arg:Int)=ev_bscissors()
  def bpaper(arg:Int)=ev_bpaper()

  if (optimize_nullary) {
    List(ev_arock,ev_brock,
	 ev_ascissors,ev_bscissors,
	 ev_apaper,ev_bpaper
       ).foreach(ev => scala.events.disjunctjoin.InEvent.markNullaryAsync(ev))
  }

  val (d1,a1,b1,b2,d2,a2,a3,b3,d3)=mkDisjunction(
    mkChord(ev_arock,   ev_brock    )  ,
    mkChord(ev_arock,   ev_bscissors )  ,
    mkChord(ev_arock,   ev_bpaper   )  ,

    mkChord(ev_ascissors,ev_brock    )  ,
    mkChord(ev_ascissors,ev_bscissors )  ,
    mkChord(ev_ascissors,ev_bpaper   )  ,

    mkChord(ev_apaper,  ev_brock    )  ,
    mkChord(ev_apaper,  ev_bscissors )  ,
    mkChord(ev_apaper,  ev_bpaper   )  )

  d1.set_optimized_async(optimize_async)

  val drw=(d1 || d2 || d3).dropParam
  val wina=(a1 || a2 || a3).dropParam
  val winb=(b1 || b2 || b3).dropParam
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)
  override def disable():Unit=d1.disable()
  def arenaName="JES_A.Unit."+asyncProvider+(if (optimize_nullary) "(opt-nullary)" else "(std-Unit)" )+(if (optimize_async) "(opt-async)" else "(std-async)")
}


class Arena_JE_S_Unit(a:Player,b:Player,runs:Int) extends Arena(a,b,runs,0) {
  val ev_arock=new ImperativeEvent[Unit]
  val ev_ascissors=new ImperativeEvent[Unit]
  val ev_apaper=new ImperativeEvent[Unit]
  val ev_brock=new ImperativeEvent[Unit]
  val ev_bscissors=new ImperativeEvent[Unit]
  val ev_bpaper=new ImperativeEvent[Unit]

  def arock(arg:Int)=ev_arock()
  def ascissors(arg:Int)=ev_ascissors()
  def apaper(arg:Int)=ev_apaper()

  def brock(arg:Int)=ev_brock()
  def bscissors(arg:Int)=ev_bscissors()
  def bpaper(arg:Int)=ev_bpaper()


  val (d1,a1,b1,b2,d2,a2,a3,b3,d3)=mkDisjunction(
    mkChord(ev_arock,   ev_brock    )  ,
    mkChord(ev_arock,   ev_bscissors )  ,
    mkChord(ev_arock,   ev_bpaper   )  ,
    mkChord(ev_ascissors,ev_brock    )  ,
    mkChord(ev_ascissors,ev_bscissors )  ,
    mkChord(ev_ascissors,ev_bpaper   )  ,
    mkChord(ev_apaper,  ev_brock    )  ,
    mkChord(ev_apaper,  ev_bscissors )  ,
    mkChord(ev_apaper,  ev_bpaper   )  )
  val drw=(d1 || d2 || d3).dropParam
  val wina=(a1 || a2 || a3).dropParam
  val winb=(b1 || b2 || b3).dropParam
  wina += ((Unit)=>{
	  a.handle_won()
	  b.handle_lost()
	  })
  winb += ((Unit)=>{
      b.handle_won()
      a.handle_lost()
      })
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)
  override def disable():Unit=d1.disable()
//  scala.events.disjunctjoin.Disjunction.set_debug(true)
//  scala.events.disjunctjoin.Chord.set_debug(true)
//  scala.events.disjunctjoin.InEvent.set_debug(true)
// scala.events.disjunctjoin.ThreadStore.set_debug(true)

  def arenaName="JES_S.Unit"
}

class Arena_SJ_Unit(a:Player,b:Player,runs:Int) extends Arena(a,b,runs,0) with Joins {
  def display(name:String)(arg:Any)=println("display "+name+" "+arg)
  object ev_arock extends joins.events.AsyncEvent[Unit]
  object ev_ascissors extends joins.events.AsyncEvent[Unit]
  object ev_apaper extends joins.events.AsyncEvent[Unit]

  object ev_brock extends joins.events.AsyncEvent[Unit]
  object ev_bscissors extends joins.events.AsyncEvent[Unit]
  object ev_bpaper extends joins.events.AsyncEvent[Unit]

  def arock(arg:Int)=ev_arock(Unit)
  def ascissors(arg:Int)=ev_ascissors(Unit)
  def apaper(arg:Int)=ev_apaper(Unit)

  def brock(arg:Int)=ev_brock(Unit)
  def bscissors(arg:Int)=ev_bscissors(Unit)
  def bpaper(arg:Int)=ev_bpaper(Unit)

  val wina=new ImperativeEvent[Unit]
  val winb=new ImperativeEvent[Unit]
  val drw=new ImperativeEvent[Unit]

  join {
    case ev_arock(x) and1 ev_brock(y) => { drw() }
    case ev_arock(x) and2 ev_bscissors(y) => { wina() }
    case ev_arock(x) and3 ev_bpaper(y) => { wina() }

    case ev_ascissors(x) and4 ev_brock(y) => { winb() }
    case ev_ascissors(x) and1 ev_bscissors(y) => { drw() }
    case ev_ascissors(x) and1 ev_bpaper(y) => { wina() }

    case ev_apaper(x) and1 ev_brock(y) => { wina() }
    case ev_apaper(x) and1 ev_bscissors(y) => { winb() }
    case ev_apaper(x) and1 ev_bpaper(y) => { drw() }
  }
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  def arenaName="ScJ_A.Unit"
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)

}

class Arena_SJ_S_Unit(a:Player,b:Player,runs:Int) extends Arena(a,b,runs,0) with Joins {
  def display(name:String)(arg:Any)=println("display "+name+" "+arg)
  object ev_arock extends joins.events.SyncEvent[Unit,Unit]
  object ev_ascissors extends joins.events.SyncEvent[Unit,Unit]
  object ev_apaper extends joins.events.SyncEvent[Unit,Unit]

  object ev_brock extends joins.events.SyncEvent[Unit,Unit]
  object ev_bscissors extends joins.events.SyncEvent[Unit,Unit]
  object ev_bpaper extends joins.events.SyncEvent[Unit,Unit]

  def arock(arg:Int)=ev_arock(Unit)
  def ascissors(arg:Int)=ev_ascissors(Unit)
  def apaper(arg:Int)=ev_apaper(Unit)

  def brock(arg:Int)=ev_brock(Unit)
  def bscissors(arg:Int)=ev_bscissors(Unit)
  def bpaper(arg:Int)=ev_bpaper(Unit)

  val wina=new ImperativeEvent[Unit]
  val winb=new ImperativeEvent[Unit]
  val drw=new ImperativeEvent[Unit]

  join {
    case ev_arock(x) and1 ev_brock(y) => { drw() ;     ev_arock.reply(); ev_brock.reply()  }
    case ev_arock(x) and2 ev_bscissors(y) => { wina(); ev_arock.reply(); ev_bscissors.reply() }
    case ev_arock(x) and3 ev_bpaper(y) => { wina() ;   ev_arock.reply(); ev_bpaper.reply() }

    case ev_ascissors(x) and4 ev_brock(y) => { winb();   ev_ascissors.reply(); ev_brock.reply()  }
    case ev_ascissors(x) and1 ev_bscissors(y) => { drw();ev_ascissors.reply(); ev_bscissors.reply() }
    case ev_ascissors(x) and1 ev_bpaper(y) => { wina();  ev_ascissors.reply(); ev_bpaper.reply() }

    case ev_apaper(x) and1 ev_brock(y) => { wina();     ev_apaper.reply();     ev_brock.reply() }
    case ev_apaper(x) and1 ev_bscissors(y) => { winb(); ev_apaper.reply(); ev_bscissors.reply() } 
    case ev_apaper(x) and1 ev_bpaper(y) => { drw();     ev_apaper.reply();     ev_bpaper.reply()}
  }
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  def arenaName="ScJ_S.Unit"
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)

}




class Arena_SJ_Int(a:Player,b:Player,runs:Int) extends Arena(a,b,runs,0) with Joins {
  def display(name:String)(arg:Any)=println("display "+name+" "+arg)
  object ev_arock extends joins.events.AsyncEvent[Int]
  object ev_ascissors extends joins.events.AsyncEvent[Int]
  object ev_apaper extends joins.events.AsyncEvent[Int]

  object ev_brock extends joins.events.AsyncEvent[Int]
  object ev_bscissors extends joins.events.AsyncEvent[Int]
  object ev_bpaper extends joins.events.AsyncEvent[Int]

  def arock(arg:Int)=ev_arock(arg)
  def ascissors(arg:Int)=ev_ascissors(arg)
  def apaper(arg:Int)=ev_apaper(arg)

  def brock(arg:Int)=ev_brock(arg)
  def bscissors(arg:Int)=ev_bscissors(arg)
  def bpaper(arg:Int)=ev_bpaper(arg)

  val wina=new ImperativeEvent[Unit]
  val winb=new ImperativeEvent[Unit]
  val drw=new ImperativeEvent[Unit]

  join {
    case ev_arock(x) and1 ev_brock(y) => { drw() }
    case ev_arock(x) and2 ev_bscissors(y) => { wina() }
    case ev_arock(x) and3 ev_bpaper(y) => { wina() }

    case ev_ascissors(x) and4 ev_brock(y) => { winb() }
    case ev_ascissors(x) and1 ev_bscissors(y) => { drw() }
    case ev_ascissors(x) and1 ev_bpaper(y) => { wina() }

    case ev_apaper(x) and1 ev_brock(y) => { wina() }
    case ev_apaper(x) and1 ev_bscissors(y) => { winb() }
    case ev_apaper(x) and1 ev_bpaper(y) => { drw() }
  }
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  def arenaName="ScJ_A.Int"
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)

}



class Arena_SJ_S_Int(a:Player,b:Player,runs:Int) extends Arena(a,b,runs,0) with Joins {
  def display(name:String)(arg:Any)=println("display "+name+" "+arg)
  object ev_arock extends joins.events.SyncEvent[Unit,Int]
  object ev_ascissors extends joins.events.SyncEvent[Unit,Int]
  object ev_apaper extends joins.events.SyncEvent[Unit,Int]

  object ev_brock extends joins.events.SyncEvent[Unit,Int]
  object ev_bscissors extends joins.events.SyncEvent[Unit,Int]
  object ev_bpaper extends joins.events.SyncEvent[Unit,Int]

  def arock(arg:Int)=ev_arock(arg)
  def ascissors(arg:Int)=ev_ascissors(arg)
  def apaper(arg:Int)=ev_apaper(arg)

  def brock(arg:Int)=ev_brock(arg)
  def bscissors(arg:Int)=ev_bscissors(arg)
  def bpaper(arg:Int)=ev_bpaper(arg)

  val wina=new ImperativeEvent[Unit]
  val winb=new ImperativeEvent[Unit]
  val drw=new ImperativeEvent[Unit]

  join {
    case ev_arock(x) and1 ev_brock(y) => { drw() ;      ev_arock.reply();ev_brock.reply() }
    case ev_arock(x) and2 ev_bscissors(y) => { wina() ; ev_arock.reply();ev_bscissors.reply() }
    case ev_arock(x) and3 ev_bpaper(y) => { wina();     ev_arock.reply(); ev_bpaper.reply() }

    case ev_ascissors(x) and4 ev_brock(y) => { winb();  ev_ascissors.reply(); ev_brock.reply() }
    case ev_ascissors(x) and1 ev_bscissors(y) => { drw();ev_ascissors.reply();ev_bscissors.reply() }
    case ev_ascissors(x) and1 ev_bpaper(y) => { wina();  ev_ascissors.reply();ev_bpaper.reply() }

    case ev_apaper(x) and1 ev_brock(y) => { wina();    ev_apaper.reply();ev_brock.reply() }
    case ev_apaper(x) and1 ev_bscissors(y) => { winb();ev_apaper.reply();ev_bscissors.reply() }
    case ev_apaper(x) and1 ev_bpaper(y) => { drw();    ev_apaper.reply();ev_bpaper.reply() }
  }
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  def arenaName="ScJ_S.Int"
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)

}



class Arena_SJ_opt(a:Player,b:Player,runs:Int) extends Arena(a,b,runs,0) with Joins {
  def display(name:String)(arg:Any)=println("display "+name+" "+arg)
  object ev_arock extends joins.events.NullaryAsyncEvent
  object ev_ascissors extends joins.events.NullaryAsyncEvent
  object ev_apaper extends joins.events.NullaryAsyncEvent

  object ev_brock extends joins.events.NullaryAsyncEvent
  object ev_bscissors extends joins.events.NullaryAsyncEvent
  object ev_bpaper extends joins.events.NullaryAsyncEvent


  def arock(arg:Int)=ev_arock()
  def ascissors(arg:Int)=ev_ascissors()
  def apaper(arg:Int)=ev_apaper()

  def brock(arg:Int)=ev_brock()
  def bscissors(arg:Int)=ev_bscissors()
  def bpaper(arg:Int)=ev_bpaper()


  val wina=new ImperativeEvent[Unit]
  val winb=new ImperativeEvent[Unit]
  val drw=new ImperativeEvent[Unit]

  join {
    case ev_arock() and1 ev_brock() => { drw() }
    case ev_arock() and2 ev_bscissors() => { wina() }
    case ev_arock() and3 ev_bpaper() => { wina() }

    case ev_ascissors() and4 ev_brock() => { winb() }
    case ev_ascissors() and1 ev_bscissors() => { drw() }
    case ev_ascissors() and1 ev_bpaper() => { wina() }

    case ev_apaper() and1 ev_brock() => { wina() }
    case ev_apaper() and1 ev_bscissors() => { winb() }
    case ev_apaper() and1 ev_bpaper() => { drw() }
  }
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  def arenaName="ScJ_A.Opt"
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)


}



class Arena_SJ_S_opt(a:Player,b:Player,runs:Int) extends Arena(a,b,runs,0) with Joins {
  def display(name:String)(arg:Any)=println("display "+name+" "+arg)
  object ev_arock extends joins.events.NullarySyncEvent[Unit]
  object ev_ascissors extends joins.events.NullarySyncEvent[Unit]
  object ev_apaper extends joins.events.NullarySyncEvent[Unit]

  object ev_brock extends joins.events.NullarySyncEvent[Unit]
  object ev_bscissors extends joins.events.NullarySyncEvent[Unit]
  object ev_bpaper extends joins.events.NullarySyncEvent[Unit]


  def arock(arg:Int)=ev_arock()
  def ascissors(arg:Int)=ev_ascissors()
  def apaper(arg:Int)=ev_apaper()

  def brock(arg:Int)=ev_brock()
  def bscissors(arg:Int)=ev_bscissors()
  def bpaper(arg:Int)=ev_bpaper()


  val wina=new ImperativeEvent[Unit]
  val winb=new ImperativeEvent[Unit]
  val drw=new ImperativeEvent[Unit]

  join {
    case ev_arock() and1 ev_brock() => { drw() ;    ev_arock.reply();ev_brock.reply()}
    case ev_arock() and2 ev_bscissors() => { wina();ev_arock.reply();ev_bscissors.reply() }
    case ev_arock() and3 ev_bpaper() => { wina();   ev_arock.reply();ev_bpaper.reply() }

    case ev_ascissors() and4 ev_brock() => { winb();   ev_ascissors.reply();ev_brock.reply() }
    case ev_ascissors() and1 ev_bscissors() => { drw();ev_ascissors.reply();ev_bscissors.reply() }
    case ev_ascissors() and1 ev_bpaper() => { wina();  ev_ascissors.reply();ev_bpaper.reply() }

    case ev_apaper() and1 ev_brock() => { wina();     ev_apaper.reply();ev_brock.reply() }
    case ev_apaper() and1 ev_bscissors() => { winb(); ev_apaper.reply();ev_bscissors.reply() }
    case ev_apaper() and1 ev_bpaper() => { drw();     ev_apaper.reply();ev_bpaper.reply() }
  }
  wina += ((Unit)=>{a.handle_won();b.handle_lost()})
  winb += ((Unit)=>{b.handle_won();a.handle_lost()})
  def arenaName="ScJ_S.Opt"
  val move=(wina || winb || drw).dropParam
  move += ((Unit)=>{
    moveHandler()
  }:Unit)


}
