package bench_rsp.rsp

import bench_rsp.rsp.players.Player
import bench_rsp.rsp.arena.Arena

class Playing(p: Player, val max: Int) extends Thread(p.name) {
  var halting = false
  var done = false
  var count = p.arena.runs
  def halt() = { halting = true }
  override def run = while (!halting && count >= 0) {
    p.play()
    count -= 1
    done = true
  }
}

object Constants {
  var RUNS = 10000
}

object R_BenchRSP {

  def singleRun(engine: Int) = {

    val p1 = new Player("Jupiter", true)
    val p2 = new Player("Zeus", false)

    val arena = Arena.create(engine, p1, p2, Constants.RUNS)

    val players = List(new Playing(p1, Constants.RUNS), new Playing(p2, Constants.RUNS))
    val beginTime = scala.compat.Platform.currentTime
    players.foreach(p => p.start())
    Thread.sleep(200)
    while (!(arena.finished)) {
      Thread.sleep(100)
    }
    players.foreach(p => p.halt())
    arena.disable()
    val duration = (arena.endTime) - beginTime
    println("duration (engine:" + engine + ") total: %d".format(duration) + " single: "+"%.4f".format( (1000.0 * duration / Constants.RUNS)) + " " + arena.name)
    duration.toInt

  }
  def list2String(arg: List[(Int, Int)]) = arg.foldLeft("")((z, a) => z + " [" + a._1 + "]" + "%6d".format(a._2) + " ")
  def list2StringNormalized(arg: List[(Int, Int)]) = arg.foldLeft("")((z, a) => z + " [" + a._1 + "]" + "%10.2f".format((1000.0 * a._2) / Constants.RUNS) + " ")

  def average(arg: List[(Int, Int)]) = (arg.foldLeft(0.0)((z: Double, x: (Int, Int)) => (z + x._2))) / arg.length

  def makeList(value: Int, times: Int): List[Int] = {
    (1 to times).toList.map(x => value)
  }

  def measure(engine: Int) = {
    val inputs = makeList(engine, 20)
    val engines = List(0, 1, 2, 3, 4)
    val warmUp = inputs.map(x => (x, singleRun(x)))
    val results = inputs.map(x => (x, singleRun(x)))

    println("results " + list2String(results))
    val total = results.foldLeft(0)((z, x: (Int, Int)) => z + x._2)
    results
  }

  def measure_runs(runs: Int) = {
    Constants.RUNS=runs
    System.gc()
    val arenas= List(
       0,1,2,3,4,   // async
       5,6,7,8,9,    // sync
       10,11,12,13,14,15,16,17
  // various optimized on/off
     )

    val measurements=arenas.map(x =>measure(x))
  //  val measurements = List(0,1,2).map(x =>measure(x))

//    println("0 scalajoins opt,  1 scalaJoins Unit, 2 JEscala Unit,  3 ScalaJoins Int ,4 JEScala Int")
//    println("5 scalajoins sync opt,  6 scalaJoins Sync Unit, 7 JEscala Sync Unit,  8 ScalaJoins Sync Int ,10 JEScala Sync Int, 11 JEScala Async UnitOpt, 12 JEScala Async UnitOpt NoAsyncOpt, 13 JEScala Unit NoAsyncOpt")

   
    println("Total Runs :" + Constants.RUNS)
    measurements.foreach(results => println(list2String(results) + "  average :" + average(results)))
    println("normalized in microseconds per single game:")
    measurements.foreach(results => println(list2StringNormalized(results)))

    print("\"iterations\"")
    val columnheaders="iterations"::arenas.map(x=>Arena.create(x, 
                                         new Player("Jupiter", true), 
					 new Player("Zeus", false),
                                         1).arenaName)

    columnheaders.foreach( l=>print("\""+l+"\" ") ) 
    println("")
    (runs, measurements.map(results => average(results)))
  }
  def main(args: Array[String]) = {
//   val iterations=(1 to 30).toList.map (x => x*1500) 
     val iterations=List(500):::(1 to 10).toList.map (x => x*1500) ::: (4 to 8).toList.map(x=>x*5000) ::: List(50000)
    val results=iterations.map(x=>measure_runs(x))
    val output=results.foldLeft("")((z1:String,line)=>z1+(line._1)+ (line._2).foldLeft("")((z2:String,vl)=>z2+" "+vl)+"\n")
 
    // format : number of rubs + averages  sj_Opt, sj_Unit, je_Unit, sj_Int , JE_Int
    println(output)
  }

}
