package eval.bench

import scala.events.{ImperativeEvent,ImperativeAsyncEvent,AbstractImperativeEvent}
import scala.events.sugarjoin.{mkDisjunction,mkChord}
import scala.compat.Platform

abstract class FSM_base(val count:Long) {
  def display(prefix:String)(arg:Any)=println(prefix+" "+Thread.currentThread+" "+arg)

  val firstLong=((a:(Long,Long))=>a._1)
  val secondLong=((a:(Long,Long))=>a._2)
  val completed=((a:Long)=>(a>=count))

  val className={
    val cn=this.getClass().getName()
    val from=cn.lastIndexOf(".")
    if (from>0) 
      cn.substring(from+1)
    else
      cn
  }

  val up:AbstractImperativeEvent[Long]
  val down:AbstractImperativeEvent[Long]
  val done=new ImperativeEvent[Unit]

  protected val block=new ImperativeEvent[Long]
  protected val unblock=new ImperativeAsyncEvent[Unit]
  protected def finished(a:(Long,Unit)):Unit={
    val ended=Platform.currentTime
    val begin=a._1
    val timed=ended-begin
    println(className+" "+count+" in "+timed+"ms  normalized ms/1K %3.3f".format(1000.0*timed/count)+"  throughput(/s): %9.2f".format(1000.0*count/timed))
    done()
  }

  def waitUntilReady()={
    block(Platform.currentTime)
  }
}


class FSM_abs(count:Long) extends FSM_base(count) {
  val up=new ImperativeAsyncEvent[Long]
  val down=new ImperativeAsyncEvent[Long]
}


class FSMs_abs(count:Long) extends FSM_base(count) {
  val up  =new ImperativeEvent[Long]
  val down=new ImperativeEvent[Long]
}



class FsmThread(val iterations:Long,name:String,val task:(Long)=>Unit) extends Thread(name) {
  private var halted=false
  def halt(arg:Unit):Unit={halted=true}
  override def run()={
    var cycle:Long=0
    while (!halted && cycle<iterations) {
      task(cycle)
      cycle+=1
    }
  }
}

class Benchmarker_abs {
  val iterations=8
  def countModify(c:Long)=c
  def cycle(fsm:FSM_base)={
    val count=fsm.count
    val threads=List(
      new FsmThread(countModify(count),"up__",((arg:Long)=>(fsm.up(arg)))),
      new FsmThread(countModify(count),"down",((arg:Long)=>(fsm.down(arg)))))
    threads.foreach(x => {
      fsm.done += ((arg:Unit)=> x.halt() )
    })
    System.gc()
    threads.foreach(x => x.start())
    fsm.waitUntilReady()
  }
}

class Benchmarkers extends Benchmarker_abs{
  override def countModify(c:Long)=2*c
}

class Benchmarker extends Benchmarker_abs {
}
