package eval.cellular.life

// loc : 74
import scala.events.{ImperativeEvent,ImperativeAsyncEvent,Observable,EventNode}
import scala.events.sugarjoin.{mkDisjunction,mkChord}

object Cell {
  var id:Int=0
  def getId=synchronized {id+=1;id}
}


// ev: 17  handler:2  imp: 1
class Cell(val x:Int,val y:Int,val board:Board) {
  val id=Cell.getId
  def display(name:String)(arg:Any)=println(name+" "+this+" "+arg)
  private var _state:Boolean=false
  def setState(arg:Boolean)={_state=arg}
  val getState=new ImperativeAsyncEvent[Int]
  val tellState=getState map ((x:Int)=>(x,_state))
  def trigger()={
    neigh.foreach(n =>n.getState(id))
  }
  
  var neigh:List[Cell]=null
  def setNeighbors(arg:List[Cell])=if (neigh==null) { 
      	neigh=arg
      	jn =createJoin 	
      	alive += ((x:Int)=>setState(true)) 
      	dead +=  ((x:Int)=>setState(false)) 
  }
  def state=if (_state) "*" else "."
  
  var jn:EventNode[List[Boolean]]=null
  lazy val counted=jn map ((a:List[Boolean])=>a.foldLeft(0)((z,x)=>z+(if (x) 1 else 0)))

  lazy val reproduce=counted && ((arg:Int)=>((arg==3) && !_state))
  lazy val underpopulated =counted && ((arg:Int)=>(_state && (arg<2) ))
  lazy val overcrowded =counted && ((arg:Int)=>(_state && (arg>3) ))
  
  
  lazy val dead =underpopulated || overcrowded
  lazy val alive=reproduce
  
  def select(o:(Int,Boolean))=(o._1)==id
  def project(o:(Int,Boolean))=(o._2)
  
  def createJoin={ // ev 9: (8+1)
    val events=neigh.map(n=>((n.tellState) && select _ ) map project _)
     val r=mkDisjunction(mkChord(events(0),events(1),events(2),events(3),events(4),events(5),events(6),events(7) ))
     r.map ((arg:(Boolean,Boolean,Boolean,Boolean,Boolean,Boolean,Boolean,Boolean))=>
           List(arg._1,arg._2,arg._3,arg._4,arg._5,arg._6,arg._7,arg._8))
  }
  override def toString="X:"+x+",Y:"+y
}

class Board(val size:Int) {
	val indexes=(0 to size).toList
	val rnd=new scala.util.Random
	val grid=indexes.map(x => (indexes.map(y => new Cell(y,x,this))))
	def get(x:Int,y:Int)={
	  val xx1=if (x>size) x-size  else x
	  val yy1=if (y>size) y-size  else y
	  val xx2=if (xx1<0) xx1+size else xx1
	  val yy2=if (yy1<0) yy1+size else yy1
	  grid(xx2)(yy2)
	}
	def neighbors(x:Int,y:Int):List[Cell]=List(
	    get(x-1,y-1), get(x  ,y-1), get(x+1,y-1) ,get(x-1,y  ) ,             
	    get(x+1,y  ), get(x-1,y+1), get(x  ,y+1), get(x+1,y+1) )
    def foreach(f:(Cell=>Unit)):Unit=indexes.foreach(y=> indexes.foreach(x=>f(get(x,y))))
	def setNeighbors=foreach(cell=>cell.setNeighbors(neighbors(cell.x,cell.y)))
    def seed=foreach(c=>c.setState(rnd.nextBoolean))
    def show=indexes.map (r=>indexes.map(c=>get(c,r).state)).foldLeft("")((z,r)=>z+r.foldLeft("")((zz,c)=>zz+c)+"\n")
   setNeighbors
 
}


// imp :1
object R_Life {
  def main(arg:Array[String]) ={
    val b=new Board(4)
    println(b.show)
    b.seed
    for (i<-1 to 100) {
    	b.foreach(c=>c.trigger())
    	println(b.show)
    	Thread.sleep(500)
    }
  }
}
