package eval.digital.half_adder

import scala.events.{ImperativeEvent,ImperativeAsyncEvent,EventNode,EventMonitor}
import scala.events.sugarjoin.{mkChord,mkDisjunction}

//# cloc 68

// defines 3 events
class HalfAdder_je(val a:EventNode[Boolean],b:EventNode[Boolean]) {
  private val j=mkDisjunction(mkChord(a,b))
  lazy val sum=j map ((a:(Boolean,Boolean))=>((a._1) ^ (a._2)))
  lazy val carry_out:EventNode[Boolean]=j map ((a:(Boolean,Boolean))=>((a._1) & (a._2)))
  // new EventMonitor(j)
}


// defines 3 events
class FullAdder_je(val a:EventNode[Boolean],b:EventNode[Boolean],val carry_in:EventNode[Boolean]) {
  private val h1=new HalfAdder_je(a,b)
  private val h2=new HalfAdder_je(h1.sum,carry_in)
  private val c=mkDisjunction(mkChord(h1.carry_out,h2.carry_out))
  val sum=h2.sum
  val carry_out=c map ((a:(Boolean,Boolean))=>((a._1) | (a._2)))
}

// defines 8 events
class Input4(val name:String) extends Thread {
  def display(name:String)(arg:Any)={println("Input4 "+name+"  "+arg)}
  var halting=false
  val rnd=new scala.util.Random
  val createRandom=new ImperativeAsyncEvent[Int]
  val request=new ImperativeAsyncEvent[Unit]

  def filter(bit:Int)(arg:Int)=((arg & bit)!=0)
  override def run=while (!halting) {
    createRandom(rnd.nextInt(8))
    Thread.sleep(500)
  }
  lazy val requested=mkDisjunction(mkChord(createRandom,request))
  //requested.markSync(createRandom)
  //requested.set_force_async(true)
  lazy val randomNumber=requested.map ((arg:(Int,Unit))=>arg._1)
  randomNumber += display(name) _
  def halt={halting=true}
  val bit8 =randomNumber map filter(8) _
  val bit4 =randomNumber map filter(4) _
  val bit2 =randomNumber map filter(2) _
  val bit1 =randomNumber map filter(1) _
 // bit8 += display("bit8") _
  //bit4 += display("bit4") _
  //bit2 += display("bit2") _
  //bit1 += display("bit1") _
}

// defines 6 events
class Adder4(val aIn:Input4,val bIn:Input4) {
  def display(name:String)(arg:Any)={println("adder4 "+name+"  "+arg)}
  val b1=new HalfAdder_je(aIn.bit1, bIn.bit1)
  val b2=new FullAdder_je(aIn.bit2, bIn.bit2, b1.carry_out)
  val b4=new FullAdder_je(aIn.bit4, bIn.bit4, b2.carry_out)
  val b8=new FullAdder_je(aIn.bit8, bIn.bit8, b4.carry_out)


  lazy val aValBits=mkDisjunction(mkChord(aIn.bit8,aIn.bit4,aIn.bit2,aIn.bit1))
  lazy val aVal=aValBits map tuple2Int _
  aValBits += display("aVal") _


  lazy val bValBits=mkDisjunction(mkChord(bIn.bit8,bIn.bit4,bIn.bit2,bIn.bit1))
  lazy val bVal=bValBits map tuple2Int _

  lazy val resValBits=mkDisjunction(mkChord(b8.sum,b4.sum,b2.sum,b1.sum))
  lazy val resVal=resValBits map tuple2Int _

  def tuple2Int(a:(Boolean,Boolean,Boolean,Boolean))=bit2Int(a._1,a._2,a._3,a._4)
  def bit2Int(bt8:Boolean,bt4:Boolean,bt2:Boolean,bt1:Boolean)=
    List((8,bt8),(4,bt4),(2,bt2),(1,bt1)).foldLeft(0)((z,x:(Int,Boolean))=>z+(if (x._2) x._1 else 0))
  aVal += display("aVal") _
  bVal += display("bVal") _
  resVal +=  display ("resVal") _
  println("adder")
}

object R_BinaryAdder {
  scala.events.AsyncProvider.setProviderType(1)
  //scala.events.disjunctjoin.ThreadStore.set_debug(true)
  //scala.events.disjunctjoin.InEvent.set_debug(true)
  //scala.events.disjunctjoin.Chord.set_debug(true)
  //scala.events.disjunctjoin.Disjunction.set_debug(true)
  val inputA=new Input4("Input A")
  val inputB=new Input4("Input B")
  val adder4=new Adder4(inputA,inputB)

  def main(arg:Array[String])={
    println("class "+this.getClass.getName)
    inputA.start()
    inputB.start()
    while (true) {
      Thread.sleep(1000)
      inputA.request()
      inputB.request()
    }
  }

}
