package eval.digital.holding_ports

import scala.events.{EventNode,ImperativeEvent,ImperativeAsyncEvent}
import scala.events.sugarjoin.{mkDisjunction,mkChord}

//# cloc 64
//# 

class Hold[T](ev:EventNode[T],zero:T) {
  var lastVal:Option[T]= None
  
  def get:T=lastVal match {
    case Some(x) => x
    case None => zero
  }
  
  def setVal(v:T):Unit=lastVal match {
    case Some(old) => if (old!=v) setTo(v)
    case None => setTo(v) 
  }
  
  def setTo(v:T)={
    lastVal=Some(v)
  }
}

// 1 events 0 handlers  imp
class And(ev1:EventNode[Boolean],ev2:EventNode[Boolean]){
  val h1=new Hold(ev1,false)
  val h2=new Hold(ev2,false)
  
  val output=(ev1 map ((b:Boolean)=>(b && h2.get))) || (ev2 map ((b:Boolean)=>(b && h1.get)))
  
}

// 1 events 0 handlers  imp
class Or(ev1:EventNode[Boolean],ev2:EventNode[Boolean]){
  val h1=new Hold(ev1,false)
  val h2=new Hold(ev2,false)
  
  val output=(ev1 map ((b:Boolean)=>(b || h2.get))) || (ev2 map ((b:Boolean)=>(b || h1.get))) 
}

// 1 events 0 handlers  imp
class Xor(ev1:EventNode[Boolean],ev2:EventNode[Boolean]){
  val h1=new Hold(ev1,false)
  val h2=new Hold(ev2,false)
  
  val output=(ev1 map ((b:Boolean)=>(b ^ h2.get))) || (ev2 map ((b:Boolean)=>(b ^ h1.get)))
  
}

// 1 imp
class SignalGenerator(val name:String) extends Thread {
  val rnd=new scala.util.Random
  private var halting:Boolean=false
  def halt={halting=true}
  val output=new ImperativeAsyncEvent[Boolean]
  override def run=while(!halting) {
    val v=rnd.nextBoolean
    println(name+" sends "+v)
    output(v)
    Thread.sleep(500+2*rnd.nextInt(25))
  }
}

// 1 handler
// 3 + 1events   0imp
object R_LogicGates {
  val sg=List(
      new SignalGenerator("alpha")  ,
      new SignalGenerator("beta")   ,
      new SignalGenerator("gamma")  )
   val ev=sg.map(x=>x.output)

  def display(name:String)(arg:Any)=println(name+" "+arg)
  def main(arg:Array[String])={
  val s1=new Xor(ev(0),ev(1))
  val s=new Xor(s1.output,ev(2))
  val c1=new And(s1.output,ev(2))
  val c2=new And(ev(0),ev(1))
  val c=new And(c1.output,c2.output)
  val result=mkDisjunction(mkChord(c.output,s.output))
  result += display("result") _
  
  sg.foreach(s=>s.start)
  Thread.sleep(30*1000)
    sg.foreach(s=>s.halt)
  }
}


