package eval.monitor

import scala.events.{ EventNode, ImperativeEvent, ImperativeAsyncEvent, Observable }
import scala.events.sugarjoin.{ mkDisjunction, mkChord }

// cloc 80
// ev: 10  handler: 1  imp:2
class Calculator {
  val canExec = new ImperativeAsyncEvent[Unit]

  var sharedVarA: Int = 0
  var sharedVarB: Int = 0

  val add2 = Observable(_add2)
  def _add2(arg: (Int, Int)): Int = {
    println("add >")
    val a = arg._1
    val b = arg._2
    sharedVarA = a
    sharedVarB = b
    while (sharedVarB > 0) {
      sharedVarA += 1
      sharedVarB -= 1
      Thread.sleep(100)
    }
    println("add <")
    sharedVarA
  }

  val mul2 = Observable(_mul2)
  def _mul2(arg: (Int, Int)): Int = {
    val a = arg._1
    val b = arg._2
    sharedVarA = a
    sharedVarB = b-1
    while (sharedVarB > 0) {
      sharedVarA += a
      sharedVarB -= 1
      Thread.sleep(100)
    }
    sharedVarA
  }

  val begin1 = add2.before.dropParam
  val begin2 = mul2.before.dropParam
  val ready1 = add2.afterAsync.dropParam
  val ready2 = mul2.afterAsync.dropParam

  val (strt1, strt2) = mkDisjunction(
    mkChord(canExec, begin1),
    mkChord(canExec, begin2))
//  strt1.markSync(begin1)
//  strt1.markSync(begin2)

  val (end1, end2) = mkDisjunction(
    mkChord(strt1, ready1)         ,
    mkChord(strt2, ready2)         )

  val ended = (end1 || end2).dropParam
  ended += ((arg: Unit) => canExec())
  canExec()
}

class CalcClient(calc: Calculator) extends Thread {
  val rnd = new scala.util.Random
  def add(a: Int, b: Int) = calc.add2((a, b))
  def mul(a: Int, b: Int) = calc.mul2((a, b))
  var halting = false
  def halt = { halting = true }
  override def run = while (!halting) {
    val a = 1+rnd.nextInt(10)
    val b = 1+rnd.nextInt(10)
    if (rnd.nextBoolean) {
      val sum=add(a,b)
      val chk=a+b
      println("add (" + a + "," + b + ")=" + sum + " chk:"+chk)
    } else {
      val prod=mul(a,b)
      val chk=a*b
      println("mul (" + a + "," + b + ")=" + prod +"  chk:"+chk)
    }
    Thread.sleep(1000 + 200 * rnd.nextInt(10))
  }
}

object R_Monitor {
  val indexes = (1 to 6).toList
  val calc = new Calculator
  val clients = indexes.map(x => new CalcClient(calc))
  def main(arg: Array[String]) = {
    clients.foreach(c => c.start)
    Thread.sleep(30 * 1000)
    clients.foreach(c => c.halt)
  }
}
