package evaldsl.bench.empty

import scala.events.{ImperativeEvent,ImperativeAsyncEvent}
import scala.events.sugarjoin.{mkDisjunction,mkChord}
import scala.compat.Platform

// this only matches async events

class FSM(val count:Long) {
  def display(prefix:String)(arg:Any)=println(prefix+" "+arg)

  val up=new ImperativeAsyncEvent[Long]
  val down=new ImperativeAsyncEvent[Long]
  var begun:Long=0-1

  up += reg_begin _
  down += reg_begin _

  def reg_begin(arg:Long):Unit=synchronized {
    if (begun<0) {
      begun=Platform.currentTime
      up -= reg_begin _
      down -= reg_begin _
    }
  }

  val done_up=up && ((x:Long)=>(x>=count))
  val done_down=up && ((x:Long)=>(x>=count))

  done_up += finished _
  done_down += finished _

  var done=false
  def finished(arg:Long):Unit=synchronized {
    if (!done) {
      done=true
      val ended=Platform.currentTime
      val timed=ended-begun
      println("finished "+count+" in "+timed+"ms  normalized ms/1K "+(1000.0*timed/count))
      done_up -= finished _
      done_down -= finished _
    }
  }
}

class FsmThread(val iterations:Long,val task:(Long)=>Unit) extends Thread {
  private var halted=false
  def halt()={halted=true}
  override def run()={
    var cycle:Long=0
    while (!halted && cycle<=iterations) {
      task(cycle)
      cycle+=1
    }
  }
}


object R_EmptyBenchmark {
  val count=500000
  def main(args:Array[String])={
    (1 to 5).foreach(x => cycle())
  }
  def cycle()={
    val fsm=new FSM(count)
    val ths=List(
      new FsmThread(count+1,((arg:Long)=>(fsm.up(arg)))),
      new FsmThread(count+1,((arg:Long)=>(fsm.down(arg)))))
   
    ths.foreach(x => x.start())
    while (!fsm.done) // a busy wait for the next cycle
      Thread.sleep(500)
  }
}
