package evaldsl.cellullar.wolfram

//# cloc 116
//# events 11
//# imp events 7
//# handlers 3
//# trigger 7

import scala.collection.mutable.ListBuffer
import scala.events._
import scala.events.sugarjoin._
import scala.events.dsl.{TriggerDSL,ProjectionDSL}

object Elementary {
  def pattern(wolfram_code: Int) =
    List(128, 64, 32, 16, 8, 4, 2, 1).map((x) => ((wolfram_code & x) != 0))
  def input(b0: Boolean, b1: Boolean, b2: Boolean): Int =
    List((4, b0), (2, b1), (1, b2)).foldLeft(0)((z, x) => (z + (if (x._2) x._1 else 0)))
  def dump(b0: Boolean, b1: Boolean, b2: Boolean, pos: Int, res: Boolean) = {
    val input = List(b0, b1, b2).foldLeft("")((z, x) => z + (if (x) "1" else "0"))
    println("convert " + input + "   " + pos + "->" + res)
  }
  def transform(pattern: List[Boolean], pos: Int) = pattern.apply(7 - pos)
}

class Elementary(val code: Int,
  val prev: EventNode[Boolean],
  val cur: EventNode[Boolean],
  val next: EventNode[Boolean])  {
  val output = new ImperativeEvent[Boolean]

  val config = Elementary.pattern(code)
  println("Wolfram code" + code + " " + config.foldLeft("")((z, x) => z + (if (x) "1" else "0")))
  lazy val done = new ImperativeAsyncEvent[Int]
  lazy val pattern = mkDisjunction(mkChord(prev, cur, next))
  pattern += transform_pattern _

  def transform_pattern(v: (Boolean, Boolean, Boolean)): Unit = {
    val position = Elementary.input(v._1, v._2, v._3)
    val res = Elementary.transform(config, position)
    output(res)
  }
}

class Shifter[T](init: T) extends ProjectionDSL {
  def display(prefix: String)(arg: Any) = println(prefix + " " + arg)
  val input = new ImperativeAsyncEvent[T]
  private val initEvent = new ImperativeAsyncEvent[T]
  private val clock = input.dropParam
  scala.events.disjunctjoin.InEvent.markNullaryAsync(clock)
  val r1 = mkDisjunction(mkChord(input, clock))
  val r11 = r1._1 || initEvent
  val r2 = mkDisjunction(mkChord(r11, clock))

  val output = r2._1 
  initEvent(init)

}

class InputDriver(val code: Int, val len: Int) extends TriggerDSL {
  println("InputDriver(" + code + "," + len + ")")
  val resultBuffer = new ListBuffer[Boolean]
  val input = new ImperativeAsyncEvent[Boolean]
  val output = new ImperativeAsyncEvent[List[Boolean]]
  val shiftRegister1 = new Shifter[Boolean](false)
  val shiftRegister2 = new Shifter[Boolean](false)
  var bufferLen: Int = 0
  input triggers shiftRegister1.input
  shiftRegister1.output  triggers  shiftRegister2.input
  val n = new Elementary(code,
    shiftRegister2.output,
    shiftRegister1.output,
    input)
  n.output += receive _

  def receive(arg: Boolean): Unit = {
    if (bufferLen < len) {
      resultBuffer += arg
      bufferLen += 1
    } else {
      val last = 2
      resultBuffer += arg
      bufferLen += 1
      resultBuffer.remove(0, last)
      output(resultBuffer.toList)
      reset()
    }
  }
  def reset() = {
    resultBuffer.clear()
    bufferLen = 0
  }
}

class InputNode(val code: Int, val len: Int) {
  val sent = new ImperativeEvent[ListBuffer[Boolean]]
  val driver = new InputDriver(code, len + 1)
  def iterate(lst: List[Boolean]) = {
    // println("iterate " + lst.length)
    driver.input(false)
    lst.foreach(t => {
      driver.input(t)
      Thread.sleep(30) // alternative for delay would be extra disjunction
    })
    driver.input(false)
    val result = new ListBuffer[Boolean]
    sent(result)
    result.toList
  }

  val received = mkDisjunction(mkChord(driver.output, sent))
  received += receiveHandler _
  def receiveHandler(arg: Tuple2[List[Boolean], ListBuffer[Boolean]]): Unit = {
    arg._2.appendAll(arg._1)
  }
  def listAsString(lst: List[Boolean]): String =
    lst.foldLeft("") { (z, x) => z + (if (x) "*" else ".") }
}
object R_Wolfram {
  val zero8List = List(0, 0, 0, 0, 0, 0, 0, 0)
  val zeroes = zero8List ::: zero8List ::: zero8List
  val initial = zeroes ::: List(1) ::: zeroes
  var input = initial.map(x => (x != 0))
  val automaton = new InputNode(30, input.length)

  def showLine(step: Int, input: List[Boolean]) = {
    println("%03d".format(step) + " show " + automaton.listAsString(input))
  }

  def main(arg: Array[String]) = {
    //  scala.events.AsyncProvider.setProviderType(1)
    showLine(0, input)
    (1 to 22).foreach(step => {
      input = automaton.iterate(input)
      showLine(step, input)
    })
  }
}
