package evaldsl.networks.walk


import evaldsl.networks.util.AdjacencyMatrix
import scala.events.{ImperativeEvent,ImperativeAsyncEvent,EventNode,VarList}
import scala.events.sugarjoin.{mkDisjunction,mkChord}
import scala.events.dsl.ProjectionDSL

//cloc 1087+72
// events
// imper 2
// handler 3

class Node(val id:Int,val gw:GraphWalker) extends Thread with ProjectionDSL  {
  val rnd=new scala.util.Random
  var neighbors:List[Node]=null
  def setNeighbors(arg:List[Node])={if (neighbors==null) neighbors=arg }
  val reached=new ImperativeAsyncEvent[List[Node]]
  reached += ((arg:List[Node])=>println("  reached "+this+" after "+arg.length+" steps"))
  
  val progress=new ImperativeEvent[Unit]
  
  var halting:Boolean=false
  def halt={halting=true}
  override def run=while (!halting) {
    progress()
    Thread.sleep(500+20*rnd.nextInt(15))
  }
  
  val reachers=new VarList[EventNode[(Unit,List[Node])]]
  def connect=neighbors.length match {
     case 1 => {  // a node with a single arc could be an end node
      val e1 =mkDisjunction(
          mkChord(progress,neighbors(0).reached))
       //e1.markSync(progress)
       reachers += e1
    }
    case 2 => {
      val (e1,e2) =mkDisjunction(
          mkChord(progress,neighbors(0).reached) ,
          mkChord(progress,neighbors(1).reached) )
       //e1.markSync(progress)
       reachers += e1
       reachers += e2
    }
    case 3 => {
      val (e1,e2,e3) =mkDisjunction(
          mkChord(progress,neighbors(0).reached) ,
          mkChord(progress,neighbors(1).reached) ,
          mkChord(progress,neighbors(2).reached) )
           //e1.markSync(progress)
       reachers += e1
       reachers += e2
       reachers += e3
    }    
    case 4 => {
      val (e1,e2,e3,e4) =mkDisjunction(
          mkChord(progress,neighbors(0).reached) ,
          mkChord(progress,neighbors(1).reached) ,
          mkChord(progress,neighbors(1).reached) ,
          mkChord(progress,neighbors(2).reached) )
           //e1.markSync(progress)
       reachers += e1
       reachers += e2
       reachers += e3
       reachers += e4
    } 
  }
  val received_history_raw=reachers.any(r => r)
  val reached_with_history=received_history_raw._2
  val reached_full_path=reached_with_history && ((arg:List[Node])=>arg.length==gw.size)
  val can_proceed=reached_with_history && ((arg:List[Node])=>(!arg.contains(this)))
  reached_full_path += printFoundPath _
  can_proceed += proceed _
  def printFoundPath(history:List[Node]):Unit={
    println("Found "+history.foldLeft("")((z,x)=>z+x))
  }
  
  def proceed(history:List[Node]):Unit={
    reached(this ::history)
  }
  
  override def start={
    connect
    super.start
  }
  override def toString()="Node["+id+"] "
}

class GraphWalker(val size:Int) {
  val rnd=new scala.util.Random
  def neighbors(id:Int)=m.listConnections(id).map((x)=>nodes(x))
  val indexes=(0 to size).toList
  val nodes=indexes.map(x=>new Node(x,this))
  val m=new AdjacencyMatrix(size)
  def init={
    m.createConnection(2,3)
    nodes.foreach(n=>n.setNeighbors(neighbors(n.id)))
  }
  def dump={
    val lines=m.dump 
    lines.foreach(l=>print("  "+l))
    }
  def start={nodes.foreach(x => x.start)}
  def halt={nodes.foreach(x => x.halt)}
  def flush={nodes.foreach(x => x.reached(Nil))}
  def trigger=nodes(rnd.nextInt(size)).reached(Nil)

}

object R_GraphWalker {
  val gw=new GraphWalker(6)
  def now=scala.compat.Platform.currentTime
  def main(arg:Array[String])={
    gw.init
    gw.dump
    val beginTime=now
    gw.start
    while (now-beginTime<50*1000) {
      gw.trigger
      Thread.sleep(2*1000)
    } 
    gw.halt
    gw.flush
    Thread.sleep(400)
    gw.dump
   }
  
}
