package evaldsl.petri.bus.st

//#cloc 49
//#events 12
//#imper 6
//#handler 9
//#trigger 8


// http://www.jfsowa.com/figs/bustop.gif
import scala.events.{ImperativeEvent,ImperativeAsyncEvent,EventNode}
import scala.events.sugarjoin._
import scala.events.dsl.TriggerDSL
// clock 49
// events 12   handlers  9  imp 2  threads 1

class Petri_JL extends TriggerDSL {
  def display(name:String)(arg:Any)=println(name+" "+arg)
  val personWaiting=new ImperativeAsyncEvent[Unit]
  val busArriving=new ImperativeAsyncEvent[Unit]


  // cycle in dataflow programming are already a problem for FrTime etc
  val onePersonGetsOnBus_delay=new ImperativeAsyncEvent[Unit]
  val busStops_delay=new ImperativeAsyncEvent[Unit]
  val busStarts_delay=new ImperativeAsyncEvent[Unit]


  val progress=new ImperativeEvent[Unit]

  // the transitions
  val (onePersonGetsOnBus,busStops,busStarts)=mkDisjunction(
    mkChord(progress,personWaiting,busWaiting),
    mkChord(progress,busArriving),
    mkChord(progress,busWaiting))

  onePersonGetsOnBus triggers onePersonGetsOnBus_delay
  busStops triggers busStops_delay
  busStarts triggers busStarts_delay

  lazy val personOnBus = onePersonGetsOnBus_delay
  lazy val busWaiting= (onePersonGetsOnBus_delay ||  busStops_delay).dropParam

  lazy  val busLeaving=busStarts

  personWaiting += display("personWaiting") _
  personOnBus += display("personOnBus") _
  busArriving += display("busArriving") _
  busWaiting += display("busWaiting") _
  busLeaving += display("busLeaving") _


  onePersonGetsOnBus += display("Join onePersonGetsOnBus") _
  busStops           += display("Join busStops") _
  busStarts          += display("Join busStarts") _

  progress += display("progress") _

  def setup_initial_tokens={
    personWaiting()
    personWaiting()
    personWaiting()
    busArriving()
    println("Initial tokens placed")
  }
  setup_initial_tokens
}



object R_PetriBus {
  val petri=new Petri_JL
  def main(arg:Array[String])={
    while (true) {
      petri.progress() // make a single step
      Thread.sleep(800)
    }
  }
}
