package evaldsl.petri.movement

import scala.events.{ImperativeEvent,ImperativeAsyncEvent}
import scala.events.sugarjoin.{mkDisjunction,mkChord}
import scala.events.dsl.TriggerDSL

//# cloc 75
//# events  16 
//# imper 4
//# handlers 6
//# trigger 6

// 'game' based on petri nets 
// a kind or '3Tennis' when considering the player as a ball
// the player can always move to 2 positions in a triangle

// 3 places at place 1 are 2 players
// each progress tick a player can move
// too another place

class Field extends TriggerDSL {
  lazy val pos1=new ImperativeAsyncEvent[Unit]
  lazy val pos2=new ImperativeAsyncEvent[Unit]
  lazy val pos3=new ImperativeAsyncEvent[Unit]
  
  var c0:Int=0
  var c1:Int=0
  var c2:Int=0
 
  def move0(name:String,step:Int)(arg:Any)={
	  c0+=step
	  status
	  }
  def move1(name:String,step:Int)(arg:Any)={
	  c1+=step
	  status
	  }
  def move2(name:String,step:Int)(arg:Any)={
	  c2+=step
	  status  
  }
  def status=println("status "+c0+" "+c1+" "+c2)
  
  
  lazy val progress=new ImperativeEvent[Unit]
  
  lazy val (topos2a, topos3a,
            topos1a, topos3b,
            topos2b, topos1b )=mkDisjunction(
		  mkChord(pos1,progress) ,
		  mkChord(pos1,progress) ,
		  mkChord(pos2,progress) ,
		  mkChord(pos2,progress) ,
		  mkChord(pos3,progress) ,
		  mkChord(pos3,progress) )
		  
  val toPos1=topos1a || topos1b
  val toPos2=topos2a || topos2b
  val toPos3=topos3a || topos3b
  
  val fromPos1=topos2a || topos3a
  val fromPos2=topos1a || topos3b
  val fromPos3=topos2b || topos1b
  
  pos1 += move0("P1",1) _
  pos2 += move1("P2",1) _
  pos3 += move2("P3",1) _
  
  fromPos1 += move0("P1",-1) _
  fromPos2 += move1("P2",-1) _
  fromPos3 += move2("P3",-1) _
    
  toPos1  triggers pos1
  toPos2  triggers pos2
  toPos3  triggers pos3

  
  def setup={
    pos1()
    pos1()
  }
  setup
}

class Trigger(val field:Field) extends Thread {
	val rnd=new scala.util.Random()
	var halting:Boolean=false
	def halt={halting=true}
	override def run() ={
	  while (!halting) {
	    Thread.sleep(100+rnd.nextInt(100)*9)
	    field.progress()
	  }
	}
}

object R_3PlayerTennis {
	val playfield=new Field
	val triggers=List(0,1,2).map(x=>new Trigger(playfield))
	def main(arg:Array[String])={
	  triggers.foreach(t => t.start)
	  Thread.sleep(10*1000)
	  triggers.foreach(t=>t.halt)
	}
}
