package evaldsl.petri.semaphore


// Two semaphore-process based on slide 9 of
// http://www.cs.colorado.edu/~kena/classes/5828/s00/lectures/lecture09.pdf


//# cloc 52
//# events 6
//# imp 1
//# handlers 0
//# trigger 2

import scala.events.{Observable,ImperativeAsyncEvent,ImperativeEvent}
import scala.events.sugarjoin.{mkDisjunction,mkChord}
import scala.events.dsl.TriggerDSL

class CriticalSection  {
  val rnd=new scala.util.Random
  def sleep_a_bit()=Thread.sleep(20+rnd.nextInt(10)*9)
  def criticalRegion(name:String):Unit={
    val l=List(0,1,2,3,4,5,6,7,8,9).foreach(t=>{
    println(name+"\t"+t)
    sleep_a_bit
    })
  }
}

class User(val name:String,val cs:CriticalSection ) extends  Thread(name){
  val rnd=new scala.util.Random
  var halting=false
  def halt={halting=true}
  val enter=Observable(_enter)
  def _enter(nm:String):Unit={
     cs.criticalRegion(nm)
  }
  override def run()={
    while (!halting) {
      enter(name)
      Thread.sleep(200+rnd.nextInt(100)*8)
    }
  }
}

// events 6   imp 1
class Semaphore(val user1:User,val user2:User) extends TriggerDSL {
  lazy val in1=user1.enter.before
  lazy val in2=user2.enter.before
  lazy val sem=new ImperativeAsyncEvent[Unit]
  
  val (cr1,cr2)=mkDisjunction(
		  mkChord(in1,sem),
		  mkChord(in2,sem)
  	    )
  //cr1.markSync(in1)
  //cr1.markSync(in2)
  
  val leaving=user1.enter.afterAsync || user2.enter.afterAsync
  leaving triggers sem
  sem() // initialize
}

object R_Semaphore {
  val cs=new CriticalSection
  val users=List("left","right").map(name=>new User(name,cs))
  val coordination=new Semaphore(users(0),users(1))	
  def main(args:Array[String])={
    println("left and right should not interleave unless in bigger blocks")
  
    users.foreach(u=>u.start)
    Thread.sleep(30*1000)
    users.foreach(u=>u.halt)
  }

}
