package evaldsl.webserver.motivation

import scala.compat.Platform
import scala.events.{ImperativeEvent,ImperativeAsyncEvent,Observable}
import scala.events.sugarjoin.{mkDisjunction,mkChord}
import scala.events.dsl.TriggerDSL


//#cloc 98
//#events 17
//#imper 6
//#handlers 3
//#trigger 8

class OnlineBookingWebApp {
  val handle=Observable(_handle)
  def _handle(arg:Unit):Unit={
    println("OB")
  }

}


class MarketPlaceWebApp {
  val handle=Observable(_handle)
  def _handle(arg:Unit):Unit={
	   println("MP")
	 }
}


class TokenGenerator extends Thread {
	var isHalting=false
	def halt={
	   isHalting=true
	 }
	val createToken=Observable(_createToken)
	def _createToken(t:Long)={
  }
  override def run=while(!isHalting) {
    Thread.sleep(1000)
    createToken(Platform.currentTime)
  }
}

class Client(val ob:OnlineBookingWebApp,val mp:MarketPlaceWebApp) extends Thread {
  val rnd=new scala.util.Random
  var halting=false
  def halt={
    halting=true
  }
  override def run()={
    while (!halting) {
      if (rnd.nextBoolean) {
	println("sendRequest OB")
	ob.handle()
      } else {
	println("sendRequest MP")
	mp.handle()
      }
      Thread.sleep(20+rnd.nextInt(8)*10)
    }
  }
}

// ipv ev 6      decl ev 12
class RateLimiter (OB:OnlineBookingWebApp,MP:MarketPlaceWebApp,TG:TokenGenerator) extends TriggerDSL{
  def display(name:String)(arg:Any)={
    println("display ["+Platform.currentTime+"]"+name+" "+arg)
  }
  val timerTick = TG.createToken.before map ((arg:Any)=>Platform.currentTime)
  val blockOB = OB.handle.before
  val blockMP = MP.handle.before
  val requestUnblockOB=new ImperativeAsyncEvent[Unit]
  val requestUnblockMP=new ImperativeAsyncEvent[Unit]
  val innerBlockOB=new ImperativeEvent[Unit]
  val innerBlockMP=new ImperativeEvent[Unit]
  val mayUnblockOB=new ImperativeAsyncEvent[Long]
  val mayUnblockMP=new ImperativeAsyncEvent[Long]

  blockOB triggers  requestUnblockOB andThen innerBlockOB
  blockMP triggers  requestUnblockMP andThen innerBlockMP

  lazy val (dummy1,dummy2)=mkDisjunction(
	      mkChord(grantUnblockOB,innerBlockOB),
	      mkChord(grantUnblockMP,innerBlockMP))
  //dummy1.markSync(innerBlockOB)
  //dummy2.markSync(innerBlockMP)


  timerTick += display("timerTick") _

  val (_mayUnblockOB,_mayUnblockMP) = mkDisjunction(
	      mkChord(requestUnblockOB,timerTick)   ,
	      mkChord(requestUnblockMP,timerTick)   )





  def isExpired(arg:Long)=((Platform.currentTime-arg) > 5000)

  _mayUnblockOB += ((arg:(Unit,Long))=>mayUnblockOB(arg._2))
  lazy val grantUnblockOB = mayUnblockOB    && (ts => !isExpired(ts))
  val  expiredUnblockOB = mayUnblockOB && (ts => isExpired(ts))
  expiredUnblockOB triggers requestUnblockOB


  _mayUnblockMP += ((arg:(Unit,Long))=>mayUnblockMP(arg._2))
  lazy val grantUnblockMP = mayUnblockMP   && (ts => !isExpired(ts))
  val expiredUnblockMP = mayUnblockMP && (ts => isExpired(ts))
  expiredUnblockMP triggers requestUnblockMP


}


 object R_Webserver {
   val ob=new OnlineBookingWebApp
   val mp=new MarketPlaceWebApp
   val tg=new TokenGenerator
   val clients=List(
       new Client(ob,mp),
       new Client(ob,mp),
       new Client(ob,mp))

   def main(arg:Array[String])={
       val coord=new RateLimiter(ob,mp,tg)
       clients.foreach(c=>c.start())
       tg.start()
       Thread.sleep(30*1000)
       clients.foreach(c=>c.halt)
       tg.halt
     }
 }
