package evalnodsl.petri.bus.mt

//# cloc 58
//# events 11
//# imper 8
//# handler 11
//# trigger 8


// http://www.jfsowa.com/figs/bustop.gif
import scala.events.{ImperativeEvent,ImperativeAsyncEvent,EventNode}
import scala.events.sugarjoin._

// cloc 58
// events 14   handlers  11  imp 8  threads 4
class Petri_JE {
  def display(name:String)(arg:Any)=println(name+" "+arg)
  val personWaiting=new ImperativeAsyncEvent[Unit]
  val busArriving=new ImperativeAsyncEvent[Unit]

  // cycle in dataflow programming are already a problem for FrTime etc
  val onePersonGetsOnBus_delay=new ImperativeAsyncEvent[Unit]
  val busStops_delay=new ImperativeAsyncEvent[Unit]
  val busStarts_delay=new ImperativeAsyncEvent[Unit]

  val progress1=new ImperativeEvent[Unit]
  val progress2=new ImperativeEvent[Unit]
  val progress3=new ImperativeEvent[Unit]

  def progressEvents=List(progress1,progress2,progress3)

  // the transitions
  val (onePersonGetsOnBus,busStops,busStarts)=mkDisjunction(
    mkChord(progress1,personWaiting,busWaiting),
    mkChord(progress2,busArriving),
    mkChord(progress3,busWaiting))

  //busStops.set_determinism(true)

  onePersonGetsOnBus += ((arg:Any)=>onePersonGetsOnBus_delay())
  busStops += ((arg:Any)=>busStops_delay())
  busStarts += ((arg:Any)=>busStarts_delay())
  lazy val personOnBus=onePersonGetsOnBus_delay
  lazy val busWaiting =onePersonGetsOnBus_delay || busStops_delay
  lazy val busLeaving=busStarts_delay

  personWaiting += display("personWaiting") _
  personOnBus += display("personOnBus") _
  busArriving += display("busArriving") _
  busWaiting += display("busWaiting") _
  busLeaving += display("busLeaving") _


  onePersonGetsOnBus_delay += display("Join onePersonGetsOnBus") _
  busStops_delay           += display("Join busStops") _
  busStarts_delay          += display("Join busStarts") _

  //progress1 += display("progress 1") _
  //progress2 += display("progress 2") _
  //progress3 += display("progress 3") _

  def setup_initial_tokens={
    personWaiting()
    personWaiting()
    personWaiting()
    busArriving()
    println("Initial tokens placed")
  }
  setup_initial_tokens
}



class VertexThread(val action:()=>Unit)  extends Thread {
  var halting=false
  def halt={halting=true}
  override def run=while (!halting) {
    action()
    Thread.sleep(300)
  }
}

object R_PetriBus {
  val petri=new Petri_JE
  def main(arg:Array[String])={
    val vectorThreads=petri.progressEvents.map(e => new VertexThread(()=>e()))
    vectorThreads.foreach(a=>(a.start))
    Thread.sleep(50*1000)
    vectorThreads.foreach(a=>(a.halt))
  }
}
