package scala.events.dsl

import scala.collection.mutable.ListBuffer
import scala.collection.immutable.HashMap
import scala.collection.immutable.{Set,ListSet}
import scala.events.{Event,ImperativeAsyncEvent,AbstractImperativeEvent,VarList}


//
//  sealed trait State
//  object Free extends State
//  object Busy extemds State
//
//  class myClass extends scala.events.dsl.FSM[State] {
//   val block=new ImperativeAsyncEvent[Unit]
//   val unblock=new ImperativeAsyncEvent[Unit]
//   val absorbed=new ImperativeAsyncEvent[Unit]
//
//   Free -> Free on unblock triggers absorbed 
//   Free -> Busy on block
//   Busy -> Free on unblock
//
//   initialState(Free)
//  }

// all transistion can optionally trigger events with a unit argument
// all transitions cam optionally apply methods with an argument
// transitions that only differ in the 'on' argument can be combine
//  Free -> Free on (unblock, reallyUnblock) triggers (alpha,beta) apply((()=>{..}),(()=>{..}))
//  setting the initialState creates the internal machinery
// for debugging use method 'showStat' to show 'equivalent code' without trigger nor apply





trait FSM_DSL[S] {
  private val inner_transitions=new ListBuffer[MultiTransition]
  private var isFrozen:Boolean=false
  def frozen=isFrozen
  def freeze()={isFrozen=true}
  lazy val transitions={
    freeze()
    inner_transitions.toList.map(t=>t.toTransitions).flatten
  }

  // set of all used states in the FSM
  lazy val states={
    (transitions.map(t=>t.from) ++ transitions.map(t=>t.to)).toSet
  }

 
  case class Transition(from:S,action:Event[Unit],to:S,triggers:List[AbstractImperativeEvent[Unit]],fun:List[()=>Unit]) {
    lazy val onMove:List[()=>Unit]=fun ++ triggers.map(t=>(()=>t()))
  //  lazy val chordString= "mkChord("+state_event_names(from)+", "+actionEventName(action)+")"
  }
  class MultiTransition(val from:S,val actions:Set[Event[Unit]],val to:S) {
    var _triggers:List[AbstractImperativeEvent[Unit]]=List()
    var _funs:List[()=>Unit]=List()
    def setTrigger(triggers:Set[AbstractImperativeEvent[Unit]])={_triggers=_triggers ++ triggers}
    def setApply(fun: List[()=>Unit])={_funs=fun}
    def toTransitions=if (actions.isEmpty) {
      throw new IllegalArgumentException("From "+from+" to "+to+" without action event")
    } else {
      actions.map(act=>{Transition(from,act,to,_triggers,_funs)}).toList
    }
  }
  implicit def s_to_Arrow(from:S)=new Arrow(from)

  class Arrow(val from:S) {
    var  _trigger=new ListSet[AbstractImperativeEvent[Unit]]
    var _trans:Option[MultiTransition]=None
    var _to:Option[S]=None
    var _action:Set[Event[Unit]]=Set()
    var _apply:List[()=>Unit]=List()
    def ->(to:S)={
      _to=Some(to)
      current_transition
      this
    }
    
    def on(action:Event[Unit]*)={	
      _action=_action ++ action.toSet
      current_transition 
      this
    }
    def current_transition=_trans match {
      case Some(tran) => _trans
      case None =>{
	_to match {
	  case None=>None
	  case Some(to)=>{
	    if (!(_action.isEmpty)) {
	      if (frozen) 
		throw new  java.lang.IllegalStateException("states already frozen")
	      val transition=new MultiTransition(from,_action,to)
	      _trans=Some(transition)
	      inner_transitions+=transition
	      setTriggers
	      setApply
	      _trans
	    }
	  }
	}
      }
    }
    def triggers(trig:AbstractImperativeEvent[Unit]*)={
      _trigger=_trigger ++ trig.toList
      setTriggers
    }
    private def setTriggers={
      _trans match {
	case Some(t)=>t.setTrigger(_trigger)
	case None=>{}
      }
      this
    }
    def apply(fun:()=>Unit*)={
      _apply=_apply ++ fun.toList
      setApply
    }
    private def setApply={
      _trans match {
	case Some(t)=>t.setApply(_apply)
	case None=>{}
      }
      this
    }
  }

  
  // after setting the initial event
  def initialState(initialState:S):Unit

  def deploy():Unit
  def undeploy():Unit

}


