package scala.events.dsl

import scala.collection.mutable.ListBuffer
import scala.collection.immutable.HashMap
import scala.collection.immutable.{Set,ListSet}
import scala.events.{Event,ImperativeAsyncEvent,AbstractImperativeEvent,VarList}


//
//  sealed trait State
//  object Free extends State
//  object Busy extemds State
//
//  class myClass extends scala.events.dsl.FSM[State] {
//   val block=new ImperativeAsyncEvent[Unit]
//   val unblock=new ImperativeAsyncEvent[Unit]
//   val absorbed=new ImperativeAsyncEvent[Unit]
//
//   Free -> Free on unblock triggers absorbed 
//   Free -> Busy on block
//   Busy -> Free on unblock
//
//   initialState(Free)
//  }

// all transistion can optionally trigger events with a unit argument
// all transitions cam optionally apply methods with an argument
// transitions that only differ in the 'on' argument can be combine
//  Free -> Free on (unblock, reallyUnblock) triggers (alpha,beta) apply((()=>{..}),(()=>{..}))
//  setting the initialState creates the internal machinery
// for debugging use method 'showStat' to show 'equivalent code' without trigger nor apply





trait FSM_DSLm[S] {
  private val inner_transitions=new ListBuffer[Transition]
  private var isFrozen:Boolean=false
  def frozen=isFrozen
  def freeze()={isFrozen=true}
  def unFreeze()={
    _states=Set()
    isFrozen=false
  }


  def transitions={
    if (!frozen) 
      freeze()
    inner_transitions.toList
  }

  def addTransition(mt:MultiTransition)=mt.toTransitions.foreach(trans=>
    {
      inner_transitions.filter(it=>it==trans).foreach(old=>{
	inner_transitions -= old
      })
      inner_transitions += trans
      }
  )

  def delTransition(mt:MultiTransition)=mt.toTransitions.foreach(trans=>{
    if (inner_transitions.contains(trans)) {
      inner_transitions -= trans
    }
  })


  // set of all used states in the FSM
  var _states:Set[S]=Set()
  def states={
    if (_states.isEmpty)
      _states= (inner_transitions.map(t=>t.from) ++ inner_transitions.map(t=>t.to)).toSet
    _states
  }

 
  case class Transition(from:S,action:Event[Unit],to:S,triggers:List[AbstractImperativeEvent[Unit]],fun:List[()=>Unit]) {
    lazy val onMove:List[()=>Unit]=fun ++ triggers.map(t=>(()=>t()))
    override def equals(oth:Any)=oth match {
      case other:Transition => (from==other.from) && (action==other.action) && (to==other.to)
      case _ => false
    }
  }
  class MultiTransition(val from:S,val actions:Set[Event[Unit]],val to:S) {
    var _triggers:Set[AbstractImperativeEvent[Unit]]=Set()
    var _funs:Set[()=>Unit]=Set()
    def setTrigger(triggers:Set[AbstractImperativeEvent[Unit]])={
      _triggers=triggers
    }
    def setApply(fun: Set[()=>Unit])={
      _funs=fun
    }
    def toTransitions=if (actions.isEmpty) {
      throw new IllegalArgumentException("From "+from+" to "+to+" without action event")
    } else {
      actions.map(act=>{Transition(from,act,to,_triggers.toList,_funs.toList)}).toList
    }
  
    override def toString()="MultiTransition("+from+" -> "+to+" on ("+actions+") triggers("+_triggers+") apply("+_funs+")"
  }
  implicit def s_to_Arrow(from:S)=new Arrow(from)

  def remove(from:S)=new Eraser(from)

  class Eraser(val from:S) {
    var  _trigger=new ListSet[AbstractImperativeEvent[Unit]]
    var _trans:Option[MultiTransition]=None
    var _to:Option[S]=None
    var _action:Set[Event[Unit]]=Set()
    var _apply:Set[()=>Unit]=Set()
    def ->(to:S)={
      _to=Some(to)
      this
    }
    
    def on(action:Event[Unit]*)={
      _action=_action ++ action.toSet
      current_transition 
      this
    }
    def triggers(trig:AbstractImperativeEvent[Unit]*)={
      _trigger=_trigger ++ trig.toList
      current_transition 
      this
    }
    private def setTriggers={
      _trans match {
	case Some(t)=>t.setTrigger(_trigger)
	case None=>{}
      }
      this
    }
    def apply(fun:()=>Unit*)={
      _apply=_apply ++ fun.toList
      current_transition
      this
    }
    private def setApply={
      _trans match {
	case Some(t)=>t.setApply(_apply)
	case None=>{}
      }
      this
    }

 def current_transition=_trans match {
      case Some(tran) => {
	setTriggers
	setApply
	delTransition(tran)
	_trans
      }
      case None =>{
	_to match {
	  case None=>None
	  case Some(to)=>{
	    if (!(_action.isEmpty)) {
	      if (frozen) 
		throw new  java.lang.IllegalStateException("states already frozen")
	      val transition=new MultiTransition(from,_action,to)
	      _trans=Some(transition)
	      setTriggers
	      setApply
	      delTransition(transition)
	      _trans
	    }
	  }
	}
      }
    }
  }


  class Arrow(val from:S) {
    var  _trigger:Set[AbstractImperativeEvent[Unit]]=Set()
    var _trans:Option[MultiTransition]=None
    var _to:Option[S]=None
    var _action:Set[Event[Unit]]=Set()
    var _apply:Set[()=>Unit]=Set()
    def ->(to:S)={
      _to=Some(to)
      this
    }
    
    def on(action:Event[Unit]*)={	
      _action=_action ++ action.toSet
      current_transition 
      this
    }
    def current_transition=_trans match {
      case Some(tran) => {
	setTriggers
	setApply
	addTransition(tran)
	_trans
      }
      case None =>{
	_to match {
	  case None=>None
	  case Some(to)=>{
	    if (!(_action.isEmpty)) {
	      if (frozen) 
		throw new  java.lang.IllegalStateException("states already frozen")
	      val transition=new MultiTransition(from,_action,to)
	      _trans=Some(transition)
	      setTriggers
	      setApply
	      addTransition(transition)
	      _trans
	    }
	  }
	}
      }
    }
    def triggers(trig:AbstractImperativeEvent[Unit]*)={
      _trigger= trig.toSet
      current_transition
      this
    }
    private def setTriggers={
      if (!(_trigger.isEmpty)) _trans match {
	case Some(t)=>t.setTrigger(_trigger)
	case None=>{}
	}
      this
    }
    def apply(fun:()=>Unit*)={
      _apply=fun.toSet
      current_transition
      this
    }
    private def setApply={
      if (!(_apply.isEmpty)) _trans match {
	case Some(t)=>t.setApply(_apply)
	case None=>{}
      }
      this
    }
  }

  
  // after setting the initial event
  def initialState(initialState:S):Unit

  def deploy():Unit
  def undeploy():Unit
}


