package scala.events.dsl
import scala.collection.immutable.{Set,ListSet,HashMap}
import scala.collection.mutable.ListBuffer
import scala.events.{Event,ImperativeAsyncEvent,AbstractImperativeEvent}


// relies on src/scala/events/dsl/fsme_evq.scala

trait FSM_E[S] extends FSM_DSL[S] {
  lazy val enum_states=states.toVector
  lazy val events=transitions.map(t=>t.action).toSet.toVector
  lazy val eventQs=events.map(ev=> new  scala.events.FSME_EventUnitQueue(ev,this))
  lazy val eventIdxs=(0 to events.length-1).toList
 
  def fromState(s:S)={
    val t=transitions.filter(t=>(t.from==s)).foldLeft(
      new HashMap[Int,Option[(Int,Transition)]]())( 
      (z,t)=>z+( (events.indexOf(t.action),Some((enum_states.indexOf(t.to),t))) )
    )
    (0 to events.size-1).map(i=>t.getOrElse(i,None)).toVector
  }

  def fromStateList(s:S)={
    transitions.filter(t=>(t.from==s)).map(t=>events.indexOf(t.action)).toList
  }

  lazy val transitionTable=enum_states.map(s => fromState(s))
 //Exp lazy val transitionEvents=enum_states.map(s => fromStateList(s).toIterable)

  private class StateHolder(s:Int) {
    var currentState:Int=0
    var validTransitions:Vector[Option[(Int,Transition)]]=Vector()
 //Exp   var validEvents:Iterable[Int]=List()

    def moveToState(newState:Int)=synchronized {
      currentState=newState
      validTransitions=transitionTable(currentState)
 //Exp     validEvents=transitionEvents(currentState)
    }
    def move(ev_idx:Int,sync:Boolean):Boolean=synchronized {
      validTransitions(ev_idx) match {
	case None => false
	case Some((newState,transition))=>{
	  transition.onMove.foreach(f=>f())
	  moveToState(newState)
	  true
	} 
      }
    }
    moveToState(s)
  }


  private var stateHolder:Option[StateHolder]=None
  private val rnd=new scala.util.Random
  def tryAction(sync:Boolean)=synchronized {
    stateHolder match {
      case None=>{false}
      case Some(sh)=>{
	lazy val validEventIdxs=eventIdxs.foldLeft(List[Int]())(
	  (z,i)=>(if (sh.validTransitions(i).isEmpty || eventQs(i).isEmpty)
		    z else (i :: z)
		))
//Exp	lazy val validEventIdxs=sh.validEvents.foldLeft(List[Int]())((z,i)=>if (eventQs(i).isEmpty) z else (z :+ i))
	val len=validEventIdxs.size
	if (len>0) {
	  val ev=if (len==1) validEventIdxs.head else validEventIdxs(rnd.nextInt(len))
	  sh.move(ev,sync)
	  eventQs(ev).dequeue
	  true
	} else {
	  false
	}
      }
    }
  }


  // after setting the initial event
  def initialState(initialState:S)={
    if (frozen)
      throw new  java.lang.IllegalStateException("states already frozen")
    transitions // forces to frozen
    transitionTable
    eventQs
    stateHolder=Some(new StateHolder(enum_states.indexOf(initialState)))
    deploy()
  }

  def deploy():Unit={
    eventQs.foreach(v=>v.deploy())
  }
 def undeploy():Unit={
    eventQs.foreach(v=>v.deploy())
  }
}


