package scala.events.dsl

import scala.collection.mutable.ListBuffer
import scala.collection.immutable.HashMap
import scala.collection.immutable.{Set,ListSet}
import scala.events.{Event,ImperativeAsyncEvent,AbstractImperativeEvent,VarList}
import scala.events.disjunctjoin.{Chord,Disjunction}



trait FSM_J[S] extends FSM_DSL[S] {
  var _actionEventNames=new HashMap[Event[Unit],Int]
  def actionEventName(action:Event[Unit])="action_"+(if (_actionEventNames.contains(action)) {
      _actionEventNames(action)
    } else {
      val nId=_actionEventNames.size
      _actionEventNames =  _actionEventNames + ((action, nId))
      nId
    })

  // stateEvent is a hashmap by applying the state it returns an event
  lazy val stateEvent=states.foldLeft(new HashMap[S,ImperativeAsyncEvent[Unit]])((z,x)=>z+((x,new ImperativeAsyncEvent[Unit])))

  // for an optional code generator
  lazy val state_event_names=states.foldLeft(new HashMap[S,String])((z,x)=>z+{ 
    val state_name_raw=x.toString.substring(1+x.toString.lastIndexOf('.'))
    val state_name=if (state_name_raw.contains("$"))
		     state_name_raw.substring(0,state_name_raw.indexOf("$"))
		   else
		     state_name_raw
    (x,"state_"+state_name)
  })
  lazy val state_event_decls=states.map(s=>"lazy val "+state_event_names(s)+" = new scala.events.ImperativeAsyncEvent[Unit]").toList
  
  lazy val join_events_list=chordEvents.map(k=>{
    (1 to  chordEvents.filter(s=>s==k).size  ).map(it=>"to_"+state_event_names(k)+"_"+it)
  }).flatten


  lazy val (disjunction,incomingEvents,joinStrings,chordEvents)={
    val chords=transitions.map(t=>{(t,
				    new Chord(List(stateEvent(t.from),t.action)),
				    "mkChord("+state_event_names(t.from)+", "+actionEventName(t.action)+")"
				  )})
    val disjunction=new Disjunction(chords.map(t=>t._2))
    val receivingStates= transitions.map(t=>t.to).toSet
    val chordEvents=chords.map(ch=>ch._1.to)
    val intermediateEvents=receivingStates.foldLeft(new HashMap[S,VarList[Chord]])((z,x)=>z+((x,{
      val varlist=new VarList[Chord]
      chords.filter((ch:(Transition,Chord,String))=>(x==(ch._1.to))).foreach(ch=>{
	varlist += (ch._2)
        // attach a handler to trigger an optional events and functions
	ch._2+=((arg:List[_])=>{ch._1.onMove.foreach(f=>{ f.apply() })})
      })
      varlist
    })))
    intermediateEvents.keys.foreach(triggerEvent=>{
      val intermediate=intermediateEvents(triggerEvent).any(t=>t.dropParam)
      val targetEvent=stateEvent(triggerEvent)
      intermediate += ((arg:Unit)=>{targetEvent.apply()})
    })
				   (disjunction,intermediateEvents,chords.map(ch=>ch._3),chordEvents)
  }
  
  // after setting the initial event
  def initialState(initialState:S)={
    if (frozen) 
      throw new  java.lang.IllegalStateException("states already frozen")
    disjunction
    stateEvent(initialState)()
  }
  def joinStringList(lst:List[String],sep:String)=if (lst.isEmpty)  
						    "" else  lst.tail.foldLeft(lst.head)((z,x)=>{z+sep+x})
  
  def showStat={
    transitions.foreach(t=>println("transition :"+t))
    states.foreach(s=>println("state :"+s))
    stateEvent.foreach(s=>println("stateEvent "+s))
    println(disjunction)
    incomingEvents.keys.foreach(l=>{
      println("Incoming for "+l+"  "+incomingEvents(l))
    })
     println("// state events")
     println(joinStringList(state_event_decls,"\n"))
     println("// disjunction")
     println("lazy val (\n  "+joinStringList(join_events_list,"  ,\n  ")
	    +" )= mkDisjunction(\n    "
	    +joinStringList(joinStrings," , \n    ")
	  +" )")

    println("// combining events")
    incomingEvents.keys.toList.map(k=>{
      "  lazy val to_"+state_event_names(k)+" = ("+joinStringList(
      (1 to incomingEvents(k).size).map(it=>"to_"+state_event_names(k)+"_"+it).toList,"\n    || ")+"  )"}).foreach(l=>println(l))

    println("// trigger new State")
    incomingEvents.keys.toList.map(k=>{
      " to_"+state_event_names(k)+" += ((arg:(Unit,Unit))=>{ "+state_event_names(k)+"() })"}).toList.foreach(l=>println(l))
  }

  def deploy()=disjunction.enable()
  def undeploy()=disjunction.disable()
}


