package scala.events.dsl
import scala.collection.immutable.{Set,ListSet,HashMap}
import scala.collection.mutable.ListBuffer
import scala.events.{Event,ImperativeAsyncEvent,AbstractImperativeEvent}


// relies on src/scala/events/dsl/fsmn_evq.scala

trait FSM_N[S] extends FSM_DSL[S] {
  lazy val transitionTable=states.foldLeft(HashMap[S,HashMap[Event[Unit],Transition]]())(
    (z,state)=>z+((state, transitions.filter(t=>(t.from==state)).foldLeft(HashMap[Event[Unit],Transition]())(
      (z,trans)=>z+((trans.action,trans))) )))

  lazy val eventQueueTable=transitions.map(tr=>tr.action).toSet.toList.foldLeft(HashMap[Event[Unit],scala.events.FSMN_EventUnitQueue[S]]())(
    (z,t)=>(z+ ((t,new scala.events.FSMN_EventUnitQueue(t,this)) )))


  private class StateHolder(s:S) {
    var currentState:S=s
    var validTransaction:Set[Event[Unit]]=Set()
    var  asyncExec:Option[scala.events.AsyncProvider]=None

    def moveToState(newState:S)=synchronized {
      currentState=newState
      validTransaction=transitionTable(currentState).keys.toSet
    }
    def move(ev:Event[Unit],sync:Boolean):Boolean=synchronized {
      if (validTransaction.contains(ev)) {
	val transition=transitionTable(currentState)(ev)
	val actions=transition.onMove
	if (!(actions.isEmpty)) { 
	  val execActions=(()=>actions.foreach(f=>f()))
	  if (sync)
	    execActions()
	  else {
	    val executor= asyncExec match {
	      case Some(x)=>x
	      case None=>{
		val provider=scala.events.AsyncProvider.create
		asyncExec=Some(provider)
		provider
		}
	    }
	    executor.apply(execActions) 
	  }
	}
	moveToState(transition.to)
	true
      } else {
	false
      }
    }
    def halt=asyncExec match {
      case Some(x)=>{
	x.haltAsync()
	asyncExec=None
      }
      case None=> {}
    }
    moveToState(s)
    
  }


  private var stateHolder:Option[StateHolder]=None
  private val rnd=new scala.util.Random
  def tryAction(sync:Boolean)=synchronized {
    stateHolder match {
      case None=>{false}
      case Some(sh)=>{
	val validEvents=sh.validTransaction.filter(ev=>(!(eventQueueTable(ev).isEmpty))).toList
	val len=validEvents.size
	if (len>0) {
	  val ev=validEvents(rnd.nextInt(len))
	  sh.move(ev,sync)
	  eventQueueTable(ev).dequeue
	  true
	} else {
	  false
	}
      }
    }
  }

  // after setting the initial event
  def initialState(initialState:S)={
    if (frozen)
      throw new  java.lang.IllegalStateException("states already frozen")
    transitions // forces to frozen
    transitionTable
    eventQueueTable
    stateHolder=Some(new StateHolder(initialState))
    deploy()
  }

  def deploy()={
    eventQueueTable.values.foreach(v=>v.deploy())
  }
  def undeploy()={
    eventQueueTable.values.foreach(v=>v.undeploy())
    stateHolder match {
      case Some(sh)=>sh.halt
      case None=>{}
    }
  }
}


