package scala.events

// used by src/scala/events/dsl/fsmn.scala
// this needs to be in the package scala.events because += and -= for 
// sinks is only visible to that package

import scala.collection.mutable.ListBuffer
import scala.collection.mutable.Queue
import scala.events

class FSMN_EventUnitQueue[S](val ev:Event[Unit],val fsm:scala.events.dsl.FSM_N[S]) {
  val queue=new Queue[ThreadStore]
  var deployed:Boolean=false
  class ThreadStore(val sync:Boolean) {
    var abandoned:Boolean=(sync==false)
    def abandon()=if (!abandoned) synchronized {
      abandoned=true
      this.notify
    }
    def enter(sync:Boolean)={
      tryAction(sync)
      if (!abandoned) synchronized {
	while (!abandoned) {
	  this.wait(1000) 
	}
      }
    }
  }
  def dequeue={
    val ts=queue.dequeue
    ts.abandon()
  }
  def isEmpty=queue.isEmpty

  private val handler:((Int,Unit,Boolean,ListBuffer[(() => Unit, List[Event[_]])]) => Unit)=((id,_,sync,react)=>{
    if (sync) {
      react += (() => {
	val ts=new ThreadStore(sync)
	queue.enqueue(ts)
	ts.enter(sync)
      },List())
    } else {
      val ts=new ThreadStore(sync)
      ts.abandon() // do not block after entering
      queue.enqueue(ts)
      ts.enter(sync)
    }
  })
  def tryAction(sync:Boolean):Boolean={
    fsm.tryAction(sync)
  }

  def deploy()=if (!deployed) {
    ev += handler
    deployed=true
  }
  def undeploy()=if (deployed) {
    ev -= handler
    queue.foreach(ts=>ts.abandon())
    queue.clear
    deployed=false
  }
}


