package scala.events.dsl

import scala.collection.mutable.ListBuffer
import scala.collection.immutable.HashMap
import scala.collection.immutable.{Set,ListSet}
import scala.events.{Event,ImperativeAsyncEvent,AbstractImperativeEvent,VarList}






trait FSM_T[S] extends FSM_DSL[S] {
  var theInitialState:Option[S]=None
  var _actionEventNames=new HashMap[Event[Unit],Int]
  def actionEventName(action:Event[Unit])="action_"+(if (_actionEventNames.contains(action)) {
      _actionEventNames(action)
    } else {
      val nId=_actionEventNames.size
      _actionEventNames =  _actionEventNames + ((action, nId))
      nId
    })

  lazy val triggerEvents=transitions.map(t=>t.triggers).flatten.toSet
  lazy val trigger_event_names=triggerEvents.foldLeft(new HashMap[AbstractImperativeEvent[Unit],String]())(
    (z,x)=>z+((x,"trigger_event_"+z.size))
    )


  lazy val triggerFunctions=transitions.map(t=>t.fun).flatten.toSet
  lazy val trigger_function_names=triggerFunctions.foldLeft(new HashMap[()=>Unit,String]())(
    (z,x)=>z+((x,"trigger_fun_"+z.size))
    )

  lazy val transitionTargets=transitions.map(t=>t.to).toSet.map((target:S)=>{
    transitions.filter(t=>(t.to==target)).foldLeft(new HashMap[Transition,String]())(
      (z,x)=>z+((x,"to_"+state_event_names(x.to)+"_"+z.size)))
  }).foldLeft(new HashMap[Transition,String]())((z,x)=>z ++ x)


  lazy val targetEventNames=transitions.map(t=>t.to).toSet.foldLeft(new HashMap[S,String]())(
    (z,st)=>z+((st,"to_"+state_event_names(st))))


  lazy val state_event_names=states.foldLeft(new HashMap[S,String])((z,x)=>z+{ 
    val state_name_raw=x.toString.substring(1+x.toString.lastIndexOf('.'))
    val state_name=if (state_name_raw.contains("$"))
		     state_name_raw.substring(0,state_name_raw.indexOf("$"))
		   else
		     state_name_raw
    (x,"state_"+state_name)
  })
  lazy val state_event_decls=states.map(s=>"lazy val "+state_event_names(s)+" = new scala.events.ImperativeAsyncEvent[Unit]").toList
  
  lazy val chords=transitions.map(t=>((t,
				       "mkChord("+state_event_names(t.from)+", "+actionEventName(t.action)+")",
				       transitionTargets(t),
				       t.fun.map(f=>trigger_function_names(f)),
				       t.triggers.map(ev=>trigger_event_names(ev))
				     )))

  lazy val targetEventCombiners=targetEventNames.keys.foldLeft(new HashMap[S,String]())(
    (z,st)=>{z+((st,targetEventNames(st)+" = ("+
	       joinStringList(transitions.filter(tran=>(tran.to==st)).map(
		 tran=>transitionTargets(tran))," || ")+")"  ))
	     })
    
  
  // after setting the initial event
  def initialState(initialState:S):Unit={
    if (frozen) 
      throw new  java.lang.IllegalStateException("states already frozen")
    theInitialState=Some(initialState)
    chords
  }
  def joinStringList(lst:List[String],sep:String)=if (lst.isEmpty)  
						    "" else  lst.tail.foldLeft(lst.head)((z,x)=>{z+sep+x})
  
  def showStat={
    println("// begin of the FSM")
     println("// state events")
     println(joinStringList(state_event_decls,"\n"))
     println("// disjunction")
     println("lazy val (\n  "+joinStringList(chords.map(c=>c._3),"  ,\n  ")
	    +" )= mkDisjunction(\n    "
	    +joinStringList(chords.map(c=>c._2)," , \n    ")
	  +" )")

   

    println("// apply functions on transitions")
    chords.foreach(ch=>{
      if (!(ch._4.isEmpty)) {
	println(ch._3+" += ((arg:(Unit,Unit))=>{ "+joinStringList(ch._4.map(f=>f+"()"),"; ")+" })")
      }
    })
    println("// trigger events on transitions")
    chords.foreach(ch=>{
      if (!(ch._5.isEmpty)) {
	println(ch._3+" += ((arg:(Unit,Unit))=>{ "+joinStringList(ch._5.map(f=>f+"()"),"; ")+" })")
      }
    })

    println("// combine events that lead to the same states")    
    targetEventCombiners.keys.foreach(tg=>{
      println("lazy val "+targetEventCombiners(tg))})

    println("// trigger the new state after a transition")
    targetEventNames.keys.foreach(tg=>{
      println( targetEventNames(tg)+" += ((arg:(Unit,Unit))=>{ "+state_event_names(tg)+"() })")
    })
    theInitialState match {
      case Some(i)=>{
	println("// Initialize by triggering "+state_event_names(i)+" once")
	println(state_event_names(i)+"()")
      }
      case None=>{}
    }

    println("// end of the FSM")

  }

  def deploy()={}
  def undeploy()={}
}


