package scala.events.dsl

import scala.events.{Event,AbstractImperativeEvent}
import scala.reflect.runtime.universe._
import scala.language.implicitConversions


//  the goal of this DSL is to support a syntax
//  ev triggers otherimperative triggers otherasync
//
//  as an alternative for creating a handler to trigger other events
//
//  the first needs to be triggers, the events after can be appended with 'then'
//
//  the triggered events can have as argument the same type as ev or the type Unit
//  it is also possible to trigger any imperative event with a constant  eg ev(1+2)
//
// important note: TypeTag is supported by Scala-2.10
//

trait TriggerDSL {
  implicit def ev2triggerable[T](ev:Event[T])(implicit ttype:TypeTag[T])=new Triggerable(ev,ttype)

  class Triggerable[T](val received_event:Event[T],val ttype:TypeTag[T]) {
    var forwarders:List[(T)=>Unit]=List()
    var deployed:Boolean=false
    val lst_parser=new Lst_parser(this)
    class Lst_parser(val outer: Triggerable[T]) {
      def triggers[U](ev:AbstractImperativeEvent[U])(implicit t:TypeTag[U])=outer.triggers[U](ev)(t)
      def andThen[U](ev:AbstractImperativeEvent[U])(implicit t:TypeTag[U])=outer.triggers[U](ev)(t)
      def triggers(trig_ev: =>Unit)=outer.triggers(trig_ev)
      def andThen(trig_ev: =>Unit)=outer.triggers(trig_ev)
    }
    
    def triggers[U](ev:AbstractImperativeEvent[U])(implicit t:TypeTag[U]) ={
      val trigger:(T=>Unit)=(
	if (t==typeTag[Unit]) {
	  (((arg:T)=>{(ev.asInstanceOf[AbstractImperativeEvent[Unit]])()}))
	} else if (t==ttype) {  // maybe this is too strict
	  ((arg:T)=>{(ev.asInstanceOf[AbstractImperativeEvent[T]])(arg.asInstanceOf[T])})
	} else {
	  throw new IllegalArgumentException(ttype+" cannot trigger any event of type "+t)
	})
      forwarders=forwarders :+ trigger
      deploy // the deployed flag ensures it is idempotent
      lst_parser
    }

    def triggers(trig_ev: =>Unit) ={
      val trigger:(T=>Unit)= ((arg:T)=>{trig_ev})
      forwarders=forwarders :+ trigger
      deploy // the deployed flag ensures it is idempotent
      lst_parser
    }

    val forwarding_handler:(T=>Unit)=(arg)=>forwarders.foreach(f=>f(arg))
    def deploy()=if (!deployed) {
      deployed=true
      received_event += forwarding_handler
    }
    // Can we call this undeploy in a real program?
    def undeploy()=if (deployed) {
      deployed=false
      received_event -= forwarding_handler
    }
  }
}

