package scala.events.sugarjoin

import scala.collection.mutable.ListBuffer
import scala.events.{Event,ImperativeEvent}
import scala.events.disjunctjoin.{Disjunction,Chord}


// This adds syntactic sugar to define disjunctions but also the resulting
// events become Tuples instead of Lists
// combining these joins:
//   imperative evt ev1[Int]  ev2[Bool] ev3[String]
//   j1= ev1 join ev2
//   j2= ev2 join ev3
//
//   lead to this code
//   val (j1,j2)=mkDisjunction( mkChord(ev1,ev2), mkChord(ev2,ev3) )
//   event types will be : j1[(Int,Bool))     j2[(Bool,String)]
//
//  j1 and j2 both have references to the hidden Disjunction
//  (maybe to keep the garbage collector away), and to give access for the
//  methods enable and disable
//
//  The events j1 and j2 both provide a set_optimized_async method that is delegated
//  to the disjunction, 
//





object mkChord {
  def apply[A,B  ](ev1:Event[A],ev2:Event[B])=new Chord2(ev1,ev2)
  def apply[A,B,C](ev1:Event[A],ev2:Event[B],
		   ev3:Event[C])=new Chord3(ev1,ev2,ev3)
  def apply[A,B,C,D  ](ev1:Event[A],ev2:Event[B],
		       ev3:Event[C],ev4:Event[D])=new Chord4(ev1,ev2,ev3,ev4)
  def apply[A,B,C,D,E](ev1:Event[A],ev2:Event[B],
		       ev3:Event[C],ev4:Event[D],
		       ev5:Event[E])=new Chord5(ev1,ev2,ev3,ev4,ev5)
  def apply[A,B,C,D,E,F](ev1:Event[A],ev2:Event[B],
			 ev3:Event[C],ev4:Event[D],
			 ev5:Event[E],ev6:Event[F])=new Chord6(ev1,ev2,ev3,ev4,ev5,ev6)
  def apply[A,B,C,D,E,F,G](ev1:Event[A],ev2:Event[B],
			   ev3:Event[C],ev4:Event[D],
			   ev5:Event[E],ev6:Event[F],
			   ev7:Event[G])=new Chord7(ev1,ev2,ev3,ev4,ev5,ev6,ev7)
  def apply[A,B,C,D,E,F,G,H](ev1:Event[A],ev2:Event[B],
			     ev3:Event[C],ev4:Event[D],
			     ev5:Event[E],ev6:Event[F],
			     ev7:Event[G],ev8:Event[H])=new Chord8(ev1,ev2,ev3,ev4,ev5,ev6,ev7,ev8)
}

object mkDisjunction {
  def apply[A](c1:Chordx[A])=(new Disjunction1[A](c1)).events
  def apply[A,B](c1:Chordx[A],c2:Chordx[B])=(new Disjunction2[A,B](c1,c2)).events
  def apply[A,B,C](c1:Chordx[A],c2:Chordx[B],
		   c3:Chordx[C])=(new Disjunction3[A,B,C](c1,c2,c3)).events
  def apply[A,B,C,D](c1:Chordx[A],c2:Chordx[B],
		     c3:Chordx[C],c4:Chordx[D])=(new Disjunction4[A,B,C,D](c1,c2,c3,c4)).events
  def apply[A,B,C,D,E](c1:Chordx[A],c2:Chordx[B],
		       c3:Chordx[C],c4:Chordx[D],
		       c5:Chordx[E])=(new Disjunction5[A,B,C,D,E](c1,c2,c3,c4,c5)).events
  def apply[A,B,C,D,E,F](c1:Chordx[A],c2:Chordx[B],
			 c3:Chordx[C],c4:Chordx[D],
			 c5:Chordx[E],c6:Chordx[F])=(new Disjunction6[A,B,C,D,E,F](c1,c2,c3,c4,c5,c6)).events
  def apply[A,B,C,D,E,F,G](c1:Chordx[A],c2:Chordx[B],
			   c3:Chordx[C],c4:Chordx[D],
			   c5:Chordx[E],c6:Chordx[F],c7:Chordx[G])=
  (new Disjunction7[A,B,C,D,E,F,G](c1,c2,c3,c4,c5,c6,c7)).events
  def apply[A,B,C,D,E,F,G,H](c1:Chordx[A],c2:Chordx[B],
			     c3:Chordx[C],c4:Chordx[D],
			     c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H])=
  (new Disjunction8[A,B,C,D,E,F,G,H](c1,c2,c3,c4,c5,c6,c7,c8)).events
  def apply[A,B,C,D,E,F,G,H,I](c1:Chordx[A],c2:Chordx[B],
			       c3:Chordx[C],c4:Chordx[D],
			       c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],c9:Chordx[I])=
  (new Disjunction9[A,B,C,D,E,F,G,H,I](c1,c2,c3,c4,c5,c6,c7,c8,c9)).events
  
  
  
  //10
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J])=
  (new Disjunction10[A,B,C,D,E,F,G,H,I,J](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10)).events			         
  //11
  def apply[A,B,C,D,E,F,G,H,I,J,K](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K])=
  (new Disjunction11[A,B,C,D,E,F,G,H,I,J,K](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11)).events			         
  
  //12
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L])=
  (new Disjunction12[A,B,C,D,E,F,G,H,I,J,K,L](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12)).events			         
  //13
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M])=
  (new Disjunction13[A,B,C,D,E,F,G,H,I,J,K,L,M](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13)).events			         
  
  //14
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N])=
  (new Disjunction14[A,B,C,D,E,F,G,H,I,J,K,L,M,N](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14)).events			         
  
  //15	
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N],c15:Chordx[O])=
  (new Disjunction15[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14,c15)).events			         
  
  //16	
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N],c15:Chordx[O],c16:Chordx[P])=
  (new Disjunction16[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14,c15,c16)).events			         
  
  //17	
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N],c15:Chordx[O],c16:Chordx[P],
    c17:Chordx[Q])=
  (new Disjunction17[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14,c15,c16,c17)).events			         
  //18	
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N],c15:Chordx[O],c16:Chordx[P],
    c17:Chordx[Q],c18:Chordx[R])=
  (new Disjunction18[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14,c15,c16,c17,c18)).events	
  // 19	
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N],c15:Chordx[O],c16:Chordx[P],
    c17:Chordx[Q],c18:Chordx[R],c19:Chordx[S])=
  (new Disjunction19[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14,c15,c16,
    c17,c18,c19)).events	
  
  // 20	
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N],c15:Chordx[O],c16:Chordx[P],
    c17:Chordx[Q],c18:Chordx[R],c19:Chordx[S],c20:Chordx[T])=
  (new Disjunction20[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14,c15,c16,
    c17,c18,c19,c20)).events	
  // 21	               
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N],c15:Chordx[O],c16:Chordx[P],
    c17:Chordx[Q],c18:Chordx[R],c19:Chordx[S],c20:Chordx[T],
    c21:Chordx[U])=
  (new Disjunction21[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14,c15,c16,
    c17,c18,c19,c20,
    c21)).events	
  // 22		               
  def apply[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V](
    c1:Chordx[A],c2:Chordx[B],c3:Chordx[C],c4:Chordx[D],
    c5:Chordx[E],c6:Chordx[F],c7:Chordx[G],c8:Chordx[H],
    c9:Chordx[I],c10:Chordx[J],c11:Chordx[K],c12:Chordx[L],
    c13:Chordx[M],c14:Chordx[N],c15:Chordx[O],c16:Chordx[P],
    c17:Chordx[Q],c18:Chordx[R],c19:Chordx[S],c20:Chordx[T],
    c21:Chordx[U],c22:Chordx[V] )=
  (new Disjunction22[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V](
    c1  ,c2 ,c3,c4 ,
    c5 , c6, c7,c8 ,
    c9 ,c10,c11,c12,
    c13,c14,c15,c16,
    c17,c18,c19,c20,
    c21,c22)).events		        
}


// RT means ReturnTuple
abstract class Chordx[RT] extends scala.events.EventNode[RT] {
  val inner:Chord
  var disjunction_o:Option[Disjunctionx]=None 
  def set_disjunction(p:Disjunctionx)=disjunction_o match {
    case None => disjunction_o=Some(p)
    case Some(_) => throw new java.lang.IllegalStateException("Setting disjunction is only possible once")
  }

  def get_disjunction=disjunction_o match {
    case Some(x) => x
    case None => null
  }

  // needs to convert the list to the right tuple to trigger
  def receive_trigger(id:Int,args:List[_],sync:Boolean, reacts: ListBuffer[(() => Unit, Trace)]):Unit


  override def deploy={
    inner += receive_trigger _
  }

  override def undeploy={
    inner -= receive_trigger _
  }
  
  def set_optimized_async(flag:Boolean)= disjunction_o match {
    case None => throw new java.lang.IllegalStateException("no disjunction specified")
    case Some(p) => p.set_optimized_async(flag)
  }

  def set_determinism(flag:Boolean)= disjunction_o match {
    case None => throw new java.lang.IllegalStateException("no disjunction specified")
    case Some(p) => p.set_determinism(flag)
  }
  def get_determinism()= disjunction_o match {
    case None => throw new java.lang.IllegalStateException("no disjunction specified")
    case Some(p) => p.get_determinism
  }

  def set_force_sync(a:Boolean) ={
    inner.set_force_sync(a)
  }
  def set_force_async(a:Boolean) ={
    inner.set_force_async(a)
  }

  def enable():Unit= disjunction_o match {
    case None => throw new java.lang.IllegalStateException("no disjunction specified")
    case Some(p) => p.enable
  }

  def disable():Unit= disjunction_o match {
    case None => throw new java.lang.IllegalStateException("no disjunction specified")
    case Some(p) => p.disable()
  }

}


// hidden Chord class (mkCord will make the right one)
class Chord2[A,B](ev1:Event[A],
		  ev2:Event[B]) extends Chordx[Tuple2[A,B]] {
  lazy val inner=new Chord(List(ev1,ev2))

  def receive_trigger(id:Int,args:List[_],sync:Boolean, reacts: ListBuffer[(() => Unit, Trace)]):Unit={
    val v= (args(0).asInstanceOf[A] ,
	    args(1).asInstanceOf[B] )
    reactions(id, v, sync, reacts)
  }
}

class Chord3[A,B,C](ev1:Event[A],
		    ev2:Event[B],
		    ev3:Event[C]) extends Chordx[Tuple3[A,B,C]] {
  val inner=new Chord(List(ev1,ev2,ev3))
  def receive_trigger(id:Int,args:List[_],sync:Boolean, reacts: ListBuffer[(() => Unit, Trace)]):Unit={
    val v= (args(0).asInstanceOf[A] ,
	    args(1).asInstanceOf[B] ,
	    args(2).asInstanceOf[C] )
    reactions(id, v, sync, reacts)
  }

}


class Chord4[A,B,C,D](ev1:Event[A],
		      ev2:Event[B],
		      ev3:Event[C],
		      ev4:Event[D]) extends Chordx[Tuple4[A,B,C,D]] {
  val inner=new Chord(List(ev1,ev2,ev3,ev4))
  def receive_trigger(id:Int,args:List[_],sync:Boolean, reacts: ListBuffer[(() => Unit, Trace)]):Unit={
    val v= (args(0).asInstanceOf[A] ,
	    args(1).asInstanceOf[B] ,
	    args(2).asInstanceOf[C] ,
	    args(3).asInstanceOf[D] )
    reactions(id, v, sync, reacts)
  }
}


class Chord5[A,B,C,D,E](ev1:Event[A],
			ev2:Event[B],
			ev3:Event[C],
			ev4:Event[D],
			ev5:Event[E]) extends Chordx[Tuple5[A,B,C,D,E]] {
  val inner=new Chord(List(ev1,ev2,ev3,ev4,ev5))
  def receive_trigger(id:Int,args:List[_],sync:Boolean, reacts: ListBuffer[(() => Unit, Trace)]):Unit={
    val v= (args(0).asInstanceOf[A] ,
	    args(1).asInstanceOf[B] ,
	    args(2).asInstanceOf[C] ,
	    args(3).asInstanceOf[D] ,
	    args(4).asInstanceOf[E] )
    reactions(id, v, sync, reacts)
  }

}


class Chord6[A,B,C,D,E,F](ev1:Event[A],
			  ev2:Event[B],
			  ev3:Event[C],
			  ev4:Event[D],
			  ev5:Event[E],
			  ev6:Event[F]) extends Chordx[Tuple6[A,B,C,D,E,F]] {
  val inner=new Chord(List(ev1,ev2,ev3,ev4,ev5,ev6))
  def receive_trigger(id:Int,args:List[_],sync:Boolean, reacts: ListBuffer[(() => Unit, Trace)]):Unit={
    val v= (args(0).asInstanceOf[A] ,
	    args(1).asInstanceOf[B] ,
	    args(2).asInstanceOf[C] ,
	    args(3).asInstanceOf[D] ,
	    args(4).asInstanceOf[E] ,
	    args(5).asInstanceOf[F] )
    reactions(id, v, sync, reacts)
  }
}


class Chord7[A,B,C,D,E,F,G](ev1:Event[A],
			    ev2:Event[B],
			    ev3:Event[C],
			    ev4:Event[D],
			    ev5:Event[E],
			    ev6:Event[F],
			    ev7:Event[G]) extends Chordx[Tuple7[A,B,C,D,E,F,G]] {
  val inner=new Chord(List(ev1,ev2,ev3,ev4,ev5,ev6,ev7))
  def receive_trigger(id:Int,args:List[_],sync:Boolean, reacts: ListBuffer[(() => Unit, Trace)]):Unit={
    val v= (args(0).asInstanceOf[A] ,
	    args(1).asInstanceOf[B] ,
	    args(2).asInstanceOf[C] ,
	    args(3).asInstanceOf[D] ,
	    args(4).asInstanceOf[E] ,
	    args(5).asInstanceOf[F] ,
	    args(6).asInstanceOf[G] )
    reactions(id, v, sync, reacts)
  }
}


class Chord8[A,B,C,D,E,F,G,H](ev1:Event[A],
			      ev2:Event[B],
			      ev3:Event[C],
			      ev4:Event[D],
			      ev5:Event[E],
			      ev6:Event[F],
			      ev7:Event[G],
			      ev8:Event[H]) extends Chordx[Tuple8[A,B,C,D,E,F,G,H]] {
  val inner=new Chord(List(ev1,ev2,ev3,ev4,ev5,ev6,ev7,ev8))
  def receive_trigger(id:Int,args:List[_],sync:Boolean, reacts: ListBuffer[(() => Unit, Trace)]):Unit={
    val v= (args(0).asInstanceOf[A] ,
	    args(1).asInstanceOf[B] ,
	    args(2).asInstanceOf[C] ,
	    args(3).asInstanceOf[D] ,
	    args(4).asInstanceOf[E] ,
	    args(5).asInstanceOf[F] ,
	    args(6).asInstanceOf[G] ,
	    args(7).asInstanceOf[H] )
    reactions(id, v, sync, reacts)
  }
}


abstract class Disjunctionx {
  def chordList:List[(Chordx[_])]
  private def getChordList:List[(Chordx[_])]={
    val chords=chordList
    chords.foreach(x => x.set_disjunction(this))
    chords
  }
  val inner=new Disjunction(getChordList.map(x=>x.inner))
  def disable():Unit=inner.disable()
  def enable():Unit=inner.enable()
  def set_optimized_async(flag:Boolean)=inner.set_optimized_async(flag)
  def set_determinism(flag:Boolean)=inner.set_determinism(flag)
  def get_determinism=inner.get_determinism
}


// hidden disjunction classes (mkDisjunction will make the right one)
class Disjunction1[A](val c1:Chordx[A]) extends Disjunctionx {
  def chordList=List(c1)
  // inner=new Disjunction(crd)
  def events=c1
}

class Disjunction2[A,B](val c1:Chordx[A],
			val c2:Chordx[B])  extends Disjunctionx{
  def chordList=List(c1,c2)
  def events=Tuple2(c1,c2)
}

class Disjunction3[A,B,C](val c1:Chordx[A],
			  val c2:Chordx[B],
			  val c3:Chordx[C])  extends Disjunctionx{
  def chordList=List(c1,c2,c3)
  def events=Tuple3(c1,c2,c3)
}


class Disjunction4[A,B,C,D](val c1:Chordx[A],
			    val c2:Chordx[B],
			    val c3:Chordx[C],
			    val c4:Chordx[D])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4)
  def events=Tuple4(c1,c2,c3,c4)
}

class Disjunction5[A,B,C,D,E](val c1:Chordx[A],
			      val c2:Chordx[B],
			      val c3:Chordx[C],
			      val c4:Chordx[D],
			      val c5:Chordx[E])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5)
  def events=Tuple5(c1,c2,c3,c4,c5)
}

class Disjunction6[A,B,C,D,E,F](val c1:Chordx[A],
				val c2:Chordx[B],
				val c3:Chordx[C],
				val c4:Chordx[D],
				val c5:Chordx[E],
				val c6:Chordx[F])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6)
  def events=Tuple6(c1,c2,c3,c4,c5,c6)
}

class Disjunction7[A,B,C,D,E,F,G](val c1:Chordx[A],
				  val c2:Chordx[B],
				  val c3:Chordx[C],
				  val c4:Chordx[D],
				  val c5:Chordx[E],
				  val c6:Chordx[F],
				  val c7:Chordx[G])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7)
  def events=Tuple7(c1,c2,c3,c4,c5,c6,c7)
}

class Disjunction8[A,B,C,D,E,F,G,H](val c1:Chordx[A],
				    val c2:Chordx[B],
				    val c3:Chordx[C],
				    val c4:Chordx[D],
				    val c5:Chordx[E],
				    val c6:Chordx[F],
				    val c7:Chordx[G],
				    val c8:Chordx[H])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8)
  def events=Tuple8(c1,c2,c3,c4,c5,c6,c7,c8)
}

class Disjunction9[A,B,C,D,E,F,G,H,I](val c1:Chordx[A],
				      val c2:Chordx[B],
				      val c3:Chordx[C],
				      val c4:Chordx[D],
				      val c5:Chordx[E],
				      val c6:Chordx[F],
				      val c7:Chordx[G],
				      val c8:Chordx[H],
				      val c9:Chordx[I])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9)
  def events=Tuple9(c1,c2,c3,c4,c5,c6,c7,c8,c9)
}

class Disjunction10[A,B,C,D,E,F,G,H,I,J](val c1:Chordx[A],
					 val c2:Chordx[B],
					 val c3:Chordx[C],
					 val c4:Chordx[D],
					 val c5:Chordx[E],
					 val c6:Chordx[F],
					 val c7:Chordx[G],
					 val c8:Chordx[H],
					 val c9:Chordx[I],
					 val c10:Chordx[J])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10)
  def events=Tuple10(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10)
}

class Disjunction11[A,B,C,D,E,F,G,H,I,J,K](val c1:Chordx[A],
					   val c2:Chordx[B],
					   val c3:Chordx[C],
					   val c4:Chordx[D],
					   val c5:Chordx[E],
					   val c6:Chordx[F],
					   val c7:Chordx[G],
					   val c8:Chordx[H],
					   val c9:Chordx[I],
					   val c10:Chordx[J],
					   val c11:Chordx[K])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11)
  def events=Tuple11(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11)
}

class Disjunction12[A,B,C,D,E,F,G,H,I,J,K,L](val c1:Chordx[A],
					     val c2:Chordx[B],
					     val c3:Chordx[C],
					     val c4:Chordx[D],
					     val c5:Chordx[E],
					     val c6:Chordx[F],
					     val c7:Chordx[G],
					     val c8:Chordx[H],
					     val c9:Chordx[I],
					     val c10:Chordx[J],
					     val c11:Chordx[K],
					     val c12:Chordx[L])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12)
  def events=Tuple12(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12)
}


class Disjunction13[A,B,C,D,E,F,G,H,I,J,K,L,M](val c1:Chordx[A],
					       val c2:Chordx[B],
					       val c3:Chordx[C],
					       val c4:Chordx[D],
					       val c5:Chordx[E],
					       val c6:Chordx[F],
					       val c7:Chordx[G],
					       val c8:Chordx[H],
					       val c9:Chordx[I],
					       val c10:Chordx[J],
					       val c11:Chordx[K],
					       val c12:Chordx[L],
					       val c13:Chordx[M])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13)
  def events=Tuple13(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13)
}

class Disjunction14[A,B,C,D,E,F,G,H,I,J,K,L,M,N](val c1:Chordx[A],
						 val c2:Chordx[B],
						 val c3:Chordx[C],
						 val c4:Chordx[D],
						 val c5:Chordx[E],
						 val c6:Chordx[F],
						 val c7:Chordx[G],
						 val c8:Chordx[H],
						 val c9:Chordx[I],
						 val c10:Chordx[J],
						 val c11:Chordx[K],
						 val c12:Chordx[L],
						 val c13:Chordx[M],
						 val c14:Chordx[N])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14)
  def events=Tuple14(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14)
}

class Disjunction15[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O](val c1:Chordx[A],
						   val c2:Chordx[B],
						   val c3:Chordx[C],
						   val c4:Chordx[D],
						   val c5:Chordx[E],
						   val c6:Chordx[F],
						   val c7:Chordx[G],
						   val c8:Chordx[H],
						   val c9:Chordx[I],
						   val c10:Chordx[J],
						   val c11:Chordx[K],
						   val c12:Chordx[L],
						   val c13:Chordx[M],
						   val c14:Chordx[N],
						   val c15:Chordx[O])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15)
  def events=Tuple15(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15)
}

class Disjunction16[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P](val c1:Chordx[A],
						     val c2:Chordx[B],
						     val c3:Chordx[C],
						     val c4:Chordx[D],
						     val c5:Chordx[E],
						     val c6:Chordx[F],
						     val c7:Chordx[G],
						     val c8:Chordx[H],
						     val c9:Chordx[I],
						     val c10:Chordx[J],
						     val c11:Chordx[K],
						     val c12:Chordx[L],
						     val c13:Chordx[M],
						     val c14:Chordx[N],
						     val c15:Chordx[O],
						     val c16:Chordx[P])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16)
  def events=Tuple16(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16)
}

class Disjunction17[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q](val c1:Chordx[A],
						       val c2:Chordx[B],
						       val c3:Chordx[C],
						       val c4:Chordx[D],
						       val c5:Chordx[E],
						       val c6:Chordx[F],
						       val c7:Chordx[G],
						       val c8:Chordx[H],
						       val c9:Chordx[I],
						       val c10:Chordx[J],
						       val c11:Chordx[K],
						       val c12:Chordx[L],
						       val c13:Chordx[M],
						       val c14:Chordx[N],
						       val c15:Chordx[O],
						       val c16:Chordx[P],          
						       val c17:Chordx[Q])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17)
  def events=Tuple17(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17)
}

class Disjunction18[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R](val c1:Chordx[A],
							 val c2:Chordx[B],
							 val c3:Chordx[C],
							 val c4:Chordx[D],
							 val c5:Chordx[E],
							 val c6:Chordx[F],
							 val c7:Chordx[G],
							 val c8:Chordx[H],
							 val c9:Chordx[I],
							 val c10:Chordx[J],
							 val c11:Chordx[K],
							 val c12:Chordx[L],
							 val c13:Chordx[M],
							 val c14:Chordx[N],
							 val c15:Chordx[O],
							 val c16:Chordx[P],
							 val c17:Chordx[Q],
							 val c18:Chordx[R])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18)
  def events=Tuple18(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18)
}

class Disjunction19[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S](val c1:Chordx[A],
							   val c2:Chordx[B],
							   val c3:Chordx[C],
							   val c4:Chordx[D],
							   val c5:Chordx[E],
							   val c6:Chordx[F],
							   val c7:Chordx[G],
							   val c8:Chordx[H],
							   val c9:Chordx[I],
							   val c10:Chordx[J],
							   val c11:Chordx[K],
							   val c12:Chordx[L],
							   val c13:Chordx[M],
							   val c14:Chordx[N],
							   val c15:Chordx[O],
							   val c16:Chordx[P],
							   val c17:Chordx[Q],
							   val c18:Chordx[R],
							   val c19:Chordx[S])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19)
  def events=Tuple19(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19)
}

class Disjunction20[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T](val c1:Chordx[A],
							     val c2:Chordx[B],
							     val c3:Chordx[C],
							     val c4:Chordx[D],
							     val c5:Chordx[E],
							     val c6:Chordx[F],
							     val c7:Chordx[G],
							     val c8:Chordx[H],
							     val c9:Chordx[I],
							     val c10:Chordx[J],
							     val c11:Chordx[K],
							     val c12:Chordx[L],
							     val c13:Chordx[M],
							     val c14:Chordx[N],
							     val c15:Chordx[O],
							     val c16:Chordx[P],
							     val c17:Chordx[Q],
							     val c18:Chordx[R],
							     val c19:Chordx[S],
							     val c20:Chordx[T])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20)
  def events=Tuple20(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20)
}

class Disjunction21[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U](val c1:Chordx[A],
							       val c2:Chordx[B],
							       val c3:Chordx[C],
							       val c4:Chordx[D],
							       val c5:Chordx[E],
							       val c6:Chordx[F],
							       val c7:Chordx[G],
							       val c8:Chordx[H],
							       val c9:Chordx[I],
							       val c10:Chordx[J],
							       val c11:Chordx[K],
							       val c12:Chordx[L],
							       val c13:Chordx[M],
							       val c14:Chordx[N],
							       val c15:Chordx[O],
							       val c16:Chordx[P],
							       val c17:Chordx[Q],
							       val c18:Chordx[R],
							       val c19:Chordx[S],
							       val c20:Chordx[T],
							       val c21:Chordx[U])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21)
  def events=Tuple21(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21)
}

class Disjunction22[A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V](val c1:Chordx[A],
								 val c2:Chordx[B],
								 val c3:Chordx[C],
								 val c4:Chordx[D],
								 val c5:Chordx[E],
								 val c6:Chordx[F],
								 val c7:Chordx[G],
								 val c8:Chordx[H],
								 val c9:Chordx[I],
								 val c10:Chordx[J],
								 val c11:Chordx[K],
								 val c12:Chordx[L],
								 val c13:Chordx[M],
								 val c14:Chordx[N],
								 val c15:Chordx[O],
								 val c16:Chordx[P],
								 val c17:Chordx[Q],
								 val c18:Chordx[R],
								 val c19:Chordx[S],
								 val c20:Chordx[T],
								 val c21:Chordx[U],
								 val c22:Chordx[V])  extends Disjunctionx{
  def chordList=List(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22)
  def events=Tuple22(c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22)
}
