package test.andi.exp1

import scala.events.{ImperativeEvent,ImperativeAsyncEvent}
import scala.events.sugarjoin.{mkChord,mkDisjunction}
import scala.events.dsl.FSM_J

class Server(iter:Int) extends Thread {
  val rnd=new scala.util.Random()
  val init=new ImperativeAsyncEvent[Unit]
  val close=new ImperativeEvent[Unit]
  override def run=for (i<- 0 to iter) {
    Thread.sleep(100+rnd.nextInt(100)*9)
    init()
    Thread.sleep(100+rnd.nextInt(100)*9)
    close()
  }

}

class Client(iter:Int) extends Thread {
  val rnd=new scala.util.Random()
  val init=new ImperativeEvent[Unit]
  val close=new ImperativeAsyncEvent[Unit]
  override def run=for (i<- 0 to iter) {
    Thread.sleep(100+rnd.nextInt(100)*9)
    init()
    Thread.sleep(100+rnd.nextInt(100)*9)
    close()
  }
}

// begin version from whiteboard
class CL(s:Server,c:Client) {
  def display(prefix:String)(arg:Any)={println("display "+prefix+" "+arg)}

  val clientInit=c.init  // alternative for the implicit before events
  val serverInit=s.init
  val clientClose=c.close
  val serverClose=s.close

  val init=new ImperativeAsyncEvent[Unit]
  val close=new ImperativeAsyncEvent[Unit]
  val finalizer=new ImperativeAsyncEvent[Unit] // r to avoid nameclash

  val (syncInit,syncClose)=mkDisjunction(
    mkChord(clientInit,serverInit,init),
    mkChord(clientClose,serverClose,close) )
  syncInit += ((arg:Any)=>close())
  syncClose += ((arg:Any)=>finalizer())

  syncInit += display("syncInit") _
  syncClose += display("syncClose") _
  finalizer += display("finalize") _
  init()
}
// end version from whiteboard

// begin alternative
trait State
object Init extends State {}
object Open extends State {}
object Final extends State {}

class CL_alt(s:Server,c:Client) extends FSM_J[State]{
  def display(prefix:String)(arg:Any)={println("display "+prefix+" "+arg)}

  val clientInit=c.init  // alternative for the implicit before events
  val serverInit=s.init
  val clientClose=c.close
  val serverClose=s.close

  val finalizer=new ImperativeAsyncEvent[Unit] // r to avoid nameclash

  val doubleInit =mkDisjunction( mkChord(clientInit,serverInit) )
  val doubleClose=mkDisjunction( mkChord(clientClose,serverClose) )

  // next 3 lines the explicit state machine
  Init -> Open on doubleInit.dropParam apply(()=>println("init -> open"))
  Open -> Final on doubleClose.dropParam triggers finalizer apply(()=>println("open -> final"))
  initialState(Init)

  finalizer += display("finalize") _
}
// end alternative

object R_Andi1 {
  val iters=2
  val cli=new Client(iters)
  val ser=new Server(iters)
  def main(arg:Array[String])={
    val impl=if (arg.isEmpty) 1 else arg(0).toInt
    val cl= impl match {
      case 1 => new CL(ser,cli)
      case 2 => new CL_alt(ser,cli)
      case _ => {
	println("usage: "+this.getClass.getName+" [ 1 | 2 ]")
	System.exit(0)
      }
    }
    ser.start
    cli.start
  }
}





