package test.animal_sim
import scala.events.{ImperativeEvent,ImperativeAsyncEvent,Observable,EventNode}
import scala.collection.mutable.ListBuffer

import scala.events.sugarjoin.{mkDisjunction,mkChord}


trait BoardElement {
  lazy val movement=moveTo.afterAsync
  private var x:Int=0
  private var y:Int=0
  private var id:Int=0-1
  private var board:Board=null

  def setBoard(b:Board)={
    if (board!=null)
      throw new IllegalArgumentException("Can't set board twice")
    board=b
    id=board.onBoard(this)
  }
  def getBoard=board

  def getId=id
  def setX(nx:Int)={
    val diff1=nx%board.col
    val diff2=(if (diff1<0) diff1+board.col else  diff1)
    x=diff2
  }
  def setY(ny:Int)={
    val diff1=ny%board.row
    val diff2=(if (diff1<0) diff1+board.row else diff1)
    y=diff2
  }
  val moveTo=Observable(_moveTo)
  def _moveTo(nPos:(Int,Int)):Unit={
    setX(nPos._1)
    setY(nPos._2)
  }
  def getX=x
  def getY=y
  
  def posString="'"+getId+"'@("+getX+","+getY+")"

}

 
class Board(val col:Int,val row:Int) {
  type Pos=(Int,Int)
  var elm_cnt:Int=0
  def display(name:String,id:Int)(arg:Any)={}
    //println("display "+name+"("+id+") "+arg)

  val lineIndexes=(0 to row).toList
  // the line indexes are used to split the board into rows which
  // are protected by a single canChangeLine for each row
  // the targetedRequest is a requestMove for the row that it moves to
 
  // this approach allows concurrency between rows but each row is
  // still protected.  Even better would be to do it for every cell but this
  // would require a lot more events and give not that much more concurrency in general

  val canChangeLine=lineIndexes.map(i => new ImperativeAsyncEvent[Unit])
  val targetedRequest=lineIndexes.map(i => {
    (requestMove && ((a:(Pos,Pos))=>(a._2._2==i)))
  })
  val content=new ListBuffer[BoardElement]
  def onBoard(be:BoardElement)=synchronized {
    content+=be
    elm_cnt+=1
    elm_cnt
  }
  def removeFromBoard(be:BoardElement)=synchronized {
    if (content.contains(be)) {
      content -= be
      true
    } else {
      false
    }
  }

  def get(x:Int,y:Int)=content.filter(e1 => (e1.getX==x)).filter(e2=>(e2.getY==y)).headOption

  def getNeighborsAndDistance(centerX:Int,centerY:Int,range:Int)={
    val rangeSquare=range*range
    var neighbors:List[(Int,BoardElement)]=Nil
    // search in the square as a rough circle, by using the get we can connect
    // edges of the board when desired
    (0-range to range).foreach(row=>
      (0-range to range).foreach(col=>
        {
          val cell_r=row*row+col*col
          if (cell_r<=rangeSquare) {
            val x=centerX+col
            val y=centerY+row
            this.get(x,y) match {
              case Some(be) => {neighbors = (cell_r,be) :: neighbors  }
              case None => {} 
            }
          }
        }))
    neighbors.sortWith((x:(Int,BoardElement),y:(Int,BoardElement))=>(x._1)<(y._1))
  }

  def getNeighbors(centerX:Int,centerY:Int,range:Int)=getNeighborsAndDistance(centerX,centerY,range).map((elem:(Int,BoardElement))=>elem._2)

  val checkFree=lineIndexes.map(i => {
    new ImperativeAsyncEvent[Pos]
  })
  lazy val requestMove=new ImperativeEvent[(Pos,Pos)]
 
  val tryMoves=lineIndexes.map(i => new ImperativeEvent[BoardElement])

  lazy val evIsFree=checkFree.map(cf => {cf  && ((p:Pos)=>free(p._1,p._2)) })
  lazy val evIsOccupied=checkFree.map (cf => {cf  && ((p:Pos)=>(!free(p._1,p._2)))})

  lineIndexes.foreach(i=>{
    evIsFree(i) += display("evIsFree",i) _
    evIsOccupied(i) += display("evIsOccupied",i) _
  })

  def free(x:Int,y:Int)=get(x,y).isEmpty


  val requests=lineIndexes.map(i => {
    val ev=mkDisjunction( mkChord(canChangeLine(i),targetedRequest(i)) )
    ev.map((arg:(Unit,(Pos,Pos)))=>arg._2)
  })
  requests.foreach(req => {req += atRequestMove _ })

  lineIndexes.foreach(i => {
    canChangeLine(i) += display("canChangeLine",i) _
  })

  def atRequestMove(arg:(Pos,Pos)):Unit={
    val from=arg._1
    val tgt=arg._2
    get(from._1,from._2) match {
      case Some(boardElem)=>{
        checkFree(tgt._2)(tgt)
        tryMoves(tgt._2)(boardElem)
      }
      case None => {println("nothing at "+from._1+","+from._2)}
    }
    canChangeLine(tgt._2)()
  }

  val disjs=lineIndexes.map( i => mkDisjunction(
    mkChord(tryMoves(i),evIsFree(i)),
    mkChord(tryMoves(i),evIsOccupied(i)) ))

  val canMoves=disjs map (double_ev=> double_ev._1)
  val bounceds=disjs map (double_ev=> double_ev._2)

  canMoves.foreach(canMove=> {
    canMove += moveElem _
  })

  def moveElem(arg:(BoardElement,Pos)):Unit={
    val el=arg._1
    val p=arg._2
    el.moveTo(p._1,p._2)
  }

  def dump(colSep:String)={
    var t=""
    (0 to row).foreach(r=>{
      var line=""
      (0 to col).foreach(c=>{
        get(c,r) match {
          case Some(x) => line += "X" + colSep
          case None => line += "-" + colSep
        }
      })
      t+=line+"\n"
    })
    t
  }


  canChangeLine.foreach(ev => ev())
}

