
package test.rateLimiter

//  ${SCALA_HOME}/bin/scala  test.webserver.WebServer
//

import scala.compat.Platform
import scala.events.{ImperativeEvent,ImperativeAsyncEvent,Observable}
import scala.events.sugarjoin.{mkDisjunction,mkChord}


// rate limitation , the block is the implicit event triggered at the begin of a handle method
// the unblock is triggered by the timer
// FSM with 2 states
//   free + unblock -> free
//   free + block   -> busy
//   busy + unblock -> free
// free and busy need a delay because of circular dependency (cycles in dataflow  jargon)


class Timer(val interval:Int,f: ()=>Unit) extends Thread {
  var halting:Boolean=false
  override def run()={
    while (!halting) {
      f()
      Thread.sleep(interval)
    }
  }
  def halt={halting=false}
  start()
}

class Experiment {
  val offset=Platform.currentTime
  def timestamp="%-1.2f".format((Platform.currentTime-offset)/1000.0)


  def display(prefix:String)(arg:Any)=println(prefix+" [load="+loadlevel+" t="+timestamp+"s] "+arg)
  val rnd=new scala.util.Random()
 
  // simulation of changing system load
  var loadlevel:Int=0
  val levelTimer=new Timer(900,()=>loadlevel=rnd.nextInt(10))
  def highload()=(loadlevel>=7)

  // simulated event from the web application
  val block = new ImperativeEvent[Unit]
  val blockFiltered = block && ((arg:Unit)=>highload())
  blockFiltered+=display("blockFiltered") _

  // event triggered with fixed intervals
  val unblock= new ImperativeAsyncEvent[Unit]
  

  val free=new ImperativeAsyncEvent[Unit]
  val busy=new ImperativeEvent[Unit]
  lazy val free_i=free1 || free2 

  lazy val (busy_i,free1,free2)=mkDisjunction(
    mkChord(blockFiltered, free), 
    mkChord(unblock,busy)       ,
    mkChord(unblock,free)       )
  busy_i += ((arg:Any)=>busy()) // delay like
  free_i += ((arg:Any)=>free()) // delay like

  free1 += display("free1 freed") _
  busy  += display("busy") _
  free() // initialize 
}


object R_limit {
  val experiment=new Experiment()
  val unblockTimer=new Timer(400,(()=>experiment.unblock()))
  val rnd=new scala.util.Random

  def main(arg:Array[String])={
   
    (1 to 100) foreach ((i)=>{
      experiment.block()
      Thread.sleep(200+rnd.nextInt(100))
    })
    println("Wait to finish")
    unblockTimer.halt

  }
}
