package test.blockingQueue


import scala.events.{ImperativeEvent,ImperativeAsyncEvent}
import scala.events.sugarjoin.{mkDisjunction,mkChord}

// simulate the idea of java.util.concurrent.BLockingQueue
// poll is already implemented without timout
// still to do  :  offer which do not block


class BlockingQueue[T](max:Int) {
  def display(prefix:String)(arg:Any)=println("  "+prefix+"  "+arg)

  class Receiver[U] {
    val receive=new ImperativeEvent[U]
    private var last:Option[U]=None
    def get=last.get
    def isEmpty=last.isEmpty
    receive += ((arg:U)=>{last=Some(arg)})
  }


  // used(0)==empty
  private val empty=new ImperativeAsyncEvent[Int]
  private val used=new ImperativeAsyncEvent[Int]


 // free(0)==full
  private val free=new ImperativeAsyncEvent[Int]
  private val full=new ImperativeAsyncEvent[Int]

  private val stored=new ImperativeAsyncEvent[T]

  val put=new ImperativeEvent[T]
  private val rqOffer=new ImperativeEvent[(T,Receiver[Boolean])]
  private val rqtake=new ImperativeEvent[Receiver[T]]
  private val rqpoll=new ImperativeEvent[Receiver[T]]


  def offer(arg:T) {
    val r=new Receiver[Boolean]
    rqOffer((arg,r))
    !(r.isEmpty)
  }

  def take()={
    val r=new Receiver[T]
    rqtake(r)
    r.get
  }

  def poll()={ // no timeout yet
    val r=new Receiver[T]
    rqpoll(r)
    if (r.isEmpty)
      null
    else
      r.get
  }


  val  (up1,up2,upo1y,upo2y,upo1n,upo2n,down1,down2,downp1y,downp2y,downp3n,downp4n )=mkDisjunction(   // the invalid combinations are commented
    mkChord(empty,free,put),  //    mkChord(empty,full,add),
    mkChord(used,free,put),   //    mkChord(used,full,add),

    mkChord(empty,free,rqOffer),  //    mkChord(empty,full,add),
    mkChord(used,free,rqOffer),   //    mkChord(used,full,add),

    mkChord(empty,full,rqOffer),  //  
    mkChord(used,full,rqOffer),   //   

    mkChord(used,free,stored,rqtake), //    mkChord(empty,full,stored,take),
    mkChord(used,full,stored,rqtake),  //    mkChord(empty,free,stored,take),

    mkChord(used,free,stored,rqpoll), //    mkChord(empty,full,stored,take),
    mkChord(used,full,stored,rqpoll),  //    mkChord(empty,free,stored,take),

    mkChord(empty,free,rqpoll), //    mkChord(empty,full,stored,take),
    mkChord(empty,full,rqpoll)  //    mkChord(empty,free,stored,take),


    )

  
  val added=up1 || up2
  added += handle_added _

  val offered=upo1y || upo2y
  offered += handle_offered _


  val taken=down1 || down2
  taken += handle_removed _

  val polled_y=downp1y || downp2y
  polled_y += handle_removed _

  val polled_n=downp3n || downp4n
  polled_n += handle_failed_poll _

  def handle_added(arg:(Int,Int,T)) {
    val (u,d,v)=arg
    stored(v)
    used(u+1)
    if (d>1)
      free(d-1)
    else
      full(d-1)
  }

  def handle_offered(arg:(Int,Int,(T,Receiver[Boolean]))) {
    val (u,d,(v,res))=arg
    stored(v)
    used(u+1)
    if (d>1)
      free(d-1)
    else
      full(d-1)
    res.receive(true)
  }

  def handle_removed(arg:(Int,Int,T,Receiver[T])) {
    val (d,u,storage,target)=arg
    target.receive(storage)
    free(u+1)
    if (d>1)
      used(d-1)
    else
      empty(d-1)
  }

  def handle_failed_poll(arg:(Int,Int,Receiver[T])) {
    val (a,d,target)=arg
    if (a>0)
      used(a)
    else
      empty(a)
    if (d>0)
      free(d)
    else
      full(d)
  }

  
//  empty  += display("empty") _  
//  free   += display("free") _
//  full   += display("full") _
//  used   += display("used") _
//  put    += display("put") _

//  stored += display("stored") _
//  rqtake += display("rqtake") _
//  rqpoll += display("rqpoll") _

  empty(0)
  free(4)
}




class StoppableQueueThread[T](val bq:BlockingQueue[T]) extends Thread {
  private var _halted:Boolean=false
  def halted=_halted
  val rnd=new scala.util.Random
  def halt={_halted=true}
}


class Reader[T](bq:BlockingQueue[T]) extends StoppableQueueThread(bq) {
  override def run={
    while (!halted) {
      Thread.sleep(5+5*rnd.nextInt(10))
      Thread.sleep(50)
      val r=bq.take()
      println("Read "+r)
    }
    println("reader halted")
  }
}

class Poller[T](bq:BlockingQueue[T]) extends StoppableQueueThread(bq) {
  override def run={
    while (!halted) {
      Thread.sleep(1+5*rnd.nextInt(10))
      val r=bq.poll()
      println("poll "+r)
    }
    println("poller halted")
  }
}

class Writer[T](bq:BlockingQueue[T],lst:List[T]) extends StoppableQueueThread(bq) {
  override def run={
    lst.foreach(arg=>(if  (!halted) {
      Thread.sleep(5+5*rnd.nextInt(10))
      println("Write "+arg)
      bq.put(arg)
    }))
    println("writer halted")
  }
}


object R_BQ {
  val bq=new BlockingQueue[Int](4)
  def getWriter=new Writer(bq,Range(1,40).toList)

  def test_reader=(new Reader[Int](bq)  ,getWriter) 
  def test_poller=(new Poller[Int](bq)  ,getWriter) 

  def listStart(th_list:List[StoppableQueueThread[_]])=th_list.foreach(th=>th.start)
  def listHalt(th_list:List[StoppableQueueThread[_]])=th_list.foreach(th=>th.halt)


  def testRun(arg:(StoppableQueueThread[_],StoppableQueueThread[_]),time:Long)={
    val wr=arg._1
    val rd=arg._2
    val threadList=List(wr,rd)
    println("Start test run")
    listStart(threadList)
    Thread.sleep(time)
    listHalt(threadList)
    println("End test run")
  }

  def main(arg:Array[String])={
    val mode=if (arg.length>0) arg(0).toInt else 0
    if (mode==0) {
      println("argument : 1 = reader")
      println("           2 = poller")
      System.exit(0)
    }
      
    val agents=arg(0).toInt match {
      case 1=>test_reader
      case 2=>test_poller
    }
    testRun(agents,1000)
  }
}
