package test.dsl.bench

import scala.events.dsl.{FSM_J,FSM_N,FSM_E,FSM_T}



abstract class Bench(val iterations:Int,val name:String) {
  val es=new EventSource 
  private var cycle:Int=0
  val trigger:(()=>Unit)=(()=>{cycle+=1})

 
  def clear()={}

  def start:Long={
    cycle=0
    print(".")
    val tm=new Timer(name+"  "+iterations)
    tm.start
    while (cycle<iterations) {
      es.trigger()
    }
    val res=tm.stop
    clear()
    es.haltAsync()
    res
  }
 def deploy()
}

class Bench_J(iter:Int) extends Bench(iter,"joins")  with FSM_J[State] {
  top -> middle    on es.l apply(trigger)
  middle -> top    on es.r apply(trigger)
  middle -> bottom on es.l apply(trigger)
  bottom -> middle on es.r apply(trigger)
  bottom -> top on es.l apply(trigger)
  top -> bottom on es.r apply(trigger)

  initialState(top)
}



class Bench_J2(iter:Int) extends Bench(iter,"joins B2")  with FSM_J[State] {
  top -> middle     on es.r  apply(trigger)
  bottom -> middle  on es.r  apply(trigger)
  middle -> top     on es.l  apply(trigger)
  middle -> bottom  on es.d  apply(trigger)

  higher -> middle  on es.l  apply(trigger)
  deeper -> middle  on es.l  apply(trigger)
  middle -> higher  on es.r  apply(trigger)
  middle -> deeper  on es.u  apply(trigger)
  
  higher -> higher  on es.u  
  deeper -> deeper  on es.d  
  higher -> deeper  on es.d  apply(trigger)
  deeper -> higher  on es.u  apply(trigger)

  top    -> bottom on es.d  apply(trigger)
  bottom -> top    on es.u  apply(trigger)
  top    -> top    on es.u  
  bottom -> bottom on es.d 

  top    -> higher on es.l  apply(trigger)
  higher -> top    on es.r  apply(trigger)
  middle -> middle on (es.u,es.d) 

  initialState(middle)
  override def clear()=undeploy()
}


class Bench_N(iter:Int) extends Bench(iter,"no joins")  with FSM_N[State]  {
  top -> middle    on es.l apply(trigger)
  middle -> top    on es.r apply(trigger)
  middle -> bottom on es.l apply(trigger)
  bottom -> middle on es.r apply(trigger)
  bottom -> top on es.l apply(trigger)
  top -> bottom on es.r apply(trigger)

  initialState(top)
}



class Bench_N2(iter:Int) extends Bench(iter,"no joins B2")  with FSM_N[State] {
   top -> middle     on es.r  apply(trigger)
  bottom -> middle  on es.r  apply(trigger)
  middle -> top     on es.l  apply(trigger)
  middle -> bottom  on es.d  apply(trigger)

  higher -> middle  on es.l  apply(trigger)
  deeper -> middle  on es.l  apply(trigger)
  middle -> higher  on es.r  apply(trigger)
  middle -> deeper  on es.u  apply(trigger)
  
  higher -> higher  on es.u  
  deeper -> deeper  on es.d  
  higher -> deeper  on es.d  apply(trigger)
  deeper -> higher  on es.u  apply(trigger)

  top    -> bottom on es.d  apply(trigger)
  bottom -> top    on es.u  apply(trigger)
  top    -> top    on es.u  
  bottom -> bottom on es.d 

  top    -> higher on es.l  apply(trigger)
  higher -> top    on es.r  apply(trigger)
  middle -> middle on (es.u,es.d) 

  initialState(middle)
  override def clear()=undeploy()
}


class Bench_E(iter:Int) extends Bench(iter,"enumerated") with FSM_E[State]  {
  top -> middle    on es.l apply(trigger)
  middle -> top    on es.r apply(trigger)
  middle -> bottom on es.l apply(trigger)
  bottom -> middle on es.r apply(trigger)
  bottom -> top on es.l apply(trigger)
  top -> bottom on es.r apply(trigger)

  initialState(top)
}


class Bench_E2(iter:Int) extends Bench(iter,"enumerated B2")  with FSM_E[State] {
  top -> middle     on es.r  apply(trigger)
  bottom -> middle  on es.r  apply(trigger)
  middle -> top     on es.l  apply(trigger)
  middle -> bottom  on es.d  apply(trigger)

  higher -> middle  on es.l  apply(trigger)
  deeper -> middle  on es.l  apply(trigger)
  middle -> higher  on es.r  apply(trigger)
  middle -> deeper  on es.u  apply(trigger)
  
  higher -> higher  on es.u  
  deeper -> deeper  on es.d  
  higher -> deeper  on es.d  apply(trigger)
  deeper -> higher  on es.u  apply(trigger)

  top    -> bottom on es.d  apply(trigger)
  bottom -> top    on es.u  apply(trigger)
  top    -> top    on es.u  
  bottom -> bottom on es.d 

  top    -> higher on es.l  apply(trigger)
  higher -> top    on es.r  apply(trigger)
  middle -> middle on (es.u,es.d) 

  initialState(middle)
  override def clear()=undeploy()
}

class Bench_T(iter:Int) extends Bench(iter,"text generation") with FSM_T[State]  {
  top -> middle    on es.l apply(trigger)
  middle -> top    on es.r apply(trigger)
  middle -> bottom on es.l apply(trigger)
  bottom -> middle on es.r apply(trigger)
  bottom -> top on es.l apply(trigger)
  top -> bottom on es.r apply(trigger)

  initialState(top)
  override def start={
    showStat
    0
  }
}


class Bench_T2(iter:Int) extends Bench(iter,"text generation B2")  with FSM_T[State] {
  top -> middle     on es.r  apply(trigger)
  bottom -> middle  on es.r  apply(trigger)
  middle -> top     on es.l  apply(trigger)
  middle -> bottom  on es.d  apply(trigger)

  higher -> middle  on es.l  apply(trigger)
  deeper -> middle  on es.l  apply(trigger)
  middle -> higher  on es.r  apply(trigger)
  middle -> deeper  on es.u  apply(trigger)
  
  higher -> higher  on es.u  
  deeper -> deeper  on es.d  
  higher -> deeper  on es.d  apply(trigger)
  deeper -> higher  on es.u  apply(trigger)

  top    -> bottom on es.d  apply(trigger)
  bottom -> top    on es.u  apply(trigger)
  top    -> top    on es.u  
  bottom -> bottom on es.d 

  top    -> higher on es.l  apply(trigger)
  higher -> top    on es.r  apply(trigger)
  middle -> middle on (es.u,es.d) 

  initialState(middle)
  override def clear()=undeploy()
  override def start={
    showStat
    0
  }

}


object R_Bench {

  def getBench(t:Int,iter:Int):Bench=t match {
    case  1 => new Bench_J(iter)
    case  2 => new Bench_N(iter)
    case  3 => new Bench_E(iter)
    case  4 => new Bench_T(iter)
    case  5 => new Bench_J2(iter)
    case  6 => new Bench_N2(iter)
    case  7 => new Bench_E2(iter)
    case  8 => new Bench_T2(iter)
    case  9 => new FSM_State_Con(iter)
    case 10 => new FSM_Con_Actor(iter)
    case _ => throw new IllegalArgumentException("no impl for "+t)
  }
  


  def setup()={
    scala.events.disjunctjoin.Disjunction.set_debug(true)
    scala.events.disjunctjoin.Chord.set_debug(true)
  }

  def runBench(b:Bench)={
    val timing=b.start
    System.gc()
    timing
  }
  
  def main(arg:Array[String])={
    if (arg.size<2) {
      println("args: 1 2 3 4 5 6 7 8 9 10 and a number of iterations")
      sys.exit(0)
    }
    //setup()
    val t=arg(0).toInt
    val iters=arg(1).toInt
    println("type "+t+" "+iters)
    val measurements=(1 to 6).toList.map(it=> runBench(getBench(t,iters))).tail
    
    val avg=(measurements.foldLeft(0.0)((z,x)=>z+x)/measurements.length)
    println("average time :"+avg+"ms "+((((avg+50)/100).toInt)/10.0)+ "s   iterations:"+iters+" engine:"+t)
  }
}
