package evalnodsl.networks.virus_mt

import evalnodsl.networks.util.AdjacencyMatrix
import scala.collection.mutable.ListBuffer
import scala.events.{ ImperativeEvent, ImperativeAsyncEvent, EventNode, VarList }
import scala.events.sugarjoin.{ mkChord, mkDisjunction }

// clock 106+72
// ev 5+10+1 handler:3+1  imp:5
class Node(val id: Int, val percentage: Int) extends Thread {
  def display(arg: Any) = println("display " + this + " " + arg)
  val rnd = new scala.util.Random
  val progress = new ImperativeEvent[Long]
  var neighbors: List[Node] = null
  var localTime: Long = 0
  lazy val tick = progress && (x => x > localTime)
  tick += ((arg: Long) => { localTime = arg })
  def oldNeighbors = neighbors.filter(x => (x.localTime < localTime)).sortWith((a, b) => a.localTime < b.localTime)
  lazy val otherTick = progress && (x => x <= localTime)
  otherTick += (age => if (oldNeighbors.length > 0) { oldNeighbors.head.progress(age) })
  def setNeighbors(others: List[Node]) = if (neighbors == null) {
    neighbors = others
  }

  val health = new ImperativeAsyncEvent[Int]
  health += ((arg: Int) => println(this + " health " + arg))

  val infections = new VarList[EventNode[(Long, Int)]]
  def connect = {
    neighbors.length match { // 10 events
      case 0 => {}
      case 1 => {
        val e1 = mkDisjunction(
          mkChord(tick, neighbors(0).health))
        //e1.markSync(tick)
        infections += e1
      }
      case 2 => {
        val (e1, e2) = mkDisjunction(
          mkChord(tick, neighbors(0).health),
          mkChord(tick, neighbors(1).health))
        //e1.markSync(tick)
        infections += e1
        infections += e2
      }
      case 3 => {
        val (e1, e2, e3) = mkDisjunction(
          mkChord(tick, neighbors(0).health),
          mkChord(tick, neighbors(1).health),
          mkChord(tick, neighbors(2).health))
        //e1.markSync(tick)
        infections += e1
        infections += e2
        infections += e3
      }
      case 4 => {
        val (e1, e2, e3, e4) = mkDisjunction(
          mkChord(tick, neighbors(0).health),
          mkChord(tick, neighbors(1).health),
          mkChord(tick, neighbors(2).health),
          mkChord(tick, neighbors(3).health))
        //e1.markSync(tick)
        infections += e1
        infections += e2
        infections += e3
        infections += e4
      }
    }
    lazy val infected = infections.any(x => x)
    infected += cycle _
    def cycle(arg: (Long, Int)): Unit = {
      val infection = arg._2
      if (infection > 0 && rnd.nextInt(100) < percentage)
        health(10) // maybe we can have degrees of illness
      else
        health(0)
    }

  }
  def init = if (rnd.nextBoolean)
    health(10)
  else
    health(0)
  override def toString = "Node " + id + " time:" + localTime
  
  var halting:Boolean=false
  def halt={halting=true}
  override def run=while (!halting) {
	  Thread.sleep(800+20*rnd.nextInt(20))
	  progress(localTime+1) 
  }
}


// ev:0 handler:0 imp: 0 
class Container(count: Int, vulnerabilityPercentage: Int) {
  val rnd = new scala.util.Random
  val indexes = (0 to count).toList
  val matrix = new AdjacencyMatrix(count)
  matrix.createConnection(2, 3)
  println(matrix.dump)
  val nodes = indexes.map(x => new Node(x, vulnerabilityPercentage))
  indexes.foreach(x => {
    println("m " + matrix.listConnections(x))
    val others = matrix.listConnections(x).filter(n => x != n).map(i => nodes(i))
    nodes(x).setNeighbors(others.to)
  })
  println("ready")
  nodes.foreach(x => x.connect)
  nodes.foreach(x => x.init)
  def start=nodes.foreach( x=>x.start)
  def halt=nodes.foreach( x=>x.halt)
}

object R_Virus {
  val cnt = new Container(10,50)

  def main(arg: Array[String]) = {
   cnt.start
   Thread.sleep(20*1000)
   cnt.halt
  }

}
